@@ -4,9 +4,6 @@
 // not use this file except in compliance with the License. You may obtain
 // a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 
-// NOTE: Any time this file is modified, a WEBSITE ticket should be opened to sync the changes with
-// the "What is MongoDB" webpage, which the example was originally added to as part of WEBSITE-5148.
-
 package documentation_examples
 
 import (
@@ -19,10 +16,11 @@ import (
 	"testing"
 	"time"
 
-	"github.com/stretchr/testify/require"
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/primitive"
+	"go.mongodb.org/mongo-driver/internal/require"
 	"go.mongodb.org/mongo-driver/mongo"
+	"go.mongodb.org/mongo-driver/mongo/integration/mtest"
 	"go.mongodb.org/mongo-driver/mongo/options"
 	"go.mongodb.org/mongo-driver/mongo/readconcern"
 	"go.mongodb.org/mongo-driver/mongo/readpref"
@@ -41,10 +39,7 @@ func requireCursorLength(t *testing.T, cursor *mongo.Cursor, length int) {
 
 func containsKey(doc bson.Raw, key ...string) bool {
 	_, err := doc.LookupErr(key...)
-	if err != nil {
-		return false
-	}
-	return true
+	return err == nil
 }
 
 func parseDate(t *testing.T, dateString string) time.Time {
@@ -56,6 +51,7 @@ func parseDate(t *testing.T, dateString string) time.Time {
 }
 
 // InsertExamples contains examples for insert operations.
+// Appears at https://www.mongodb.com/docs/manual/tutorial/insert-documents/.
 func InsertExamples(t *testing.T, db *mongo.Database) {
 	coll := db.Collection("inventory_insert")
 
@@ -66,7 +62,7 @@ func InsertExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 1
 
 		result, err := coll.InsertOne(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"item", "canvas"},
 				{"qty", 100},
@@ -88,7 +84,7 @@ func InsertExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 2
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{{"item", "canvas"}},
 		)
 
@@ -103,7 +99,7 @@ func InsertExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 3
 
 		result, err := coll.InsertMany(
-			context.Background(),
+			context.TODO(),
 			[]interface{}{
 				bson.D{
 					{"item", "journal"},
@@ -145,6 +141,7 @@ func InsertExamples(t *testing.T, db *mongo.Database) {
 }
 
 // QueryToplevelFieldsExamples contains examples for querying top-level fields.
+// Appears at https://www.mongodb.com/docs/manual/tutorial/query-documents/.
 func QueryToplevelFieldsExamples(t *testing.T, db *mongo.Database) {
 	coll := db.Collection("inventory_query_top")
 
@@ -207,7 +204,7 @@ func QueryToplevelFieldsExamples(t *testing.T, db *mongo.Database) {
 			},
 		}
 
-		result, err := coll.InsertMany(context.Background(), docs)
+		result, err := coll.InsertMany(context.TODO(), docs)
 
 		// End Example 6
 
@@ -219,7 +216,7 @@ func QueryToplevelFieldsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 7
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{},
 		)
 
@@ -233,7 +230,7 @@ func QueryToplevelFieldsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 9
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{{"status", "D"}},
 		)
 
@@ -247,7 +244,7 @@ func QueryToplevelFieldsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 10
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{{"status", bson.D{{"$in", bson.A{"A", "D"}}}}})
 
 		// End Example 10
@@ -260,7 +257,7 @@ func QueryToplevelFieldsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 11
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"status", "A"},
 				{"qty", bson.D{{"$lt", 30}}},
@@ -276,7 +273,7 @@ func QueryToplevelFieldsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 12
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"$or",
 					bson.A{
@@ -295,7 +292,7 @@ func QueryToplevelFieldsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 13
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"status", "A"},
 				{"$or", bson.A{
@@ -313,6 +310,7 @@ func QueryToplevelFieldsExamples(t *testing.T, db *mongo.Database) {
 }
 
 // QueryEmbeddedDocumentsExamples contains examples for querying embedded document fields.
+// Appears at https://www.mongodb.com/docs/manual/tutorial/query-embedded-documents/.
 func QueryEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 	coll := db.Collection("inventory_query_embedded")
 
@@ -375,7 +373,7 @@ func QueryEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 			},
 		}
 
-		result, err := coll.InsertMany(context.Background(), docs)
+		result, err := coll.InsertMany(context.TODO(), docs)
 
 		// End Example 14
 
@@ -387,7 +385,7 @@ func QueryEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 15
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"size", bson.D{
 					{"h", 14},
@@ -406,7 +404,7 @@ func QueryEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 16
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"size", bson.D{
 					{"w", 21},
@@ -425,7 +423,7 @@ func QueryEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 17
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{{"size.uom", "in"}},
 		)
 
@@ -439,7 +437,7 @@ func QueryEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 18
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"size.h", bson.D{
 					{"$lt", 15},
@@ -456,7 +454,7 @@ func QueryEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 19
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"size.h", bson.D{
 					{"$lt", 15},
@@ -474,6 +472,7 @@ func QueryEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 }
 
 // QueryArraysExamples contains examples for querying array fields.
+// Appears at https://www.mongodb.com/docs/manual/tutorial/query-arrays/.
 func QueryArraysExamples(t *testing.T, db *mongo.Database) {
 	coll := db.Collection("inventory_query_array")
 
@@ -516,7 +515,7 @@ func QueryArraysExamples(t *testing.T, db *mongo.Database) {
 			},
 		}
 
-		result, err := coll.InsertMany(context.Background(), docs)
+		result, err := coll.InsertMany(context.TODO(), docs)
 
 		// End Example 20
 
@@ -528,7 +527,7 @@ func QueryArraysExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 21
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{{"tags", bson.A{"red", "blank"}}},
 		)
 
@@ -542,7 +541,7 @@ func QueryArraysExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 22
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"tags", bson.D{{"$all", bson.A{"red", "blank"}}}},
 			})
@@ -557,7 +556,7 @@ func QueryArraysExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 23
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"tags", "red"},
 			})
@@ -572,7 +571,7 @@ func QueryArraysExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 24
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"dim_cm", bson.D{
 					{"$gt", 25},
@@ -589,7 +588,7 @@ func QueryArraysExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 25
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"dim_cm", bson.D{
 					{"$gt", 15},
@@ -607,7 +606,7 @@ func QueryArraysExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 26
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"dim_cm", bson.D{
 					{"$elemMatch", bson.D{
@@ -627,7 +626,7 @@ func QueryArraysExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 27
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"dim_cm.1", bson.D{
 					{"$gt", 25},
@@ -644,7 +643,7 @@ func QueryArraysExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 28
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"tags", bson.D{
 					{"$size", 3},
@@ -660,6 +659,7 @@ func QueryArraysExamples(t *testing.T, db *mongo.Database) {
 }
 
 // QueryArrayEmbeddedDocumentsExamples contains examples for querying fields with arrays and embedded documents.
+// Appears at https://www.mongodb.com/docs/manual/tutorial/query-array-of-documents/.
 func QueryArrayEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 	coll := db.Collection("inventory_query_array_embedded")
 
@@ -733,7 +733,7 @@ func QueryArrayEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 			},
 		}
 
-		result, err := coll.InsertMany(context.Background(), docs)
+		result, err := coll.InsertMany(context.TODO(), docs)
 
 		// End Example 29
 
@@ -745,7 +745,7 @@ func QueryArrayEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 30
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"instock", bson.D{
 					{"warehouse", "A"},
@@ -763,7 +763,7 @@ func QueryArrayEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 31
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"instock", bson.D{
 					{"qty", 5},
@@ -781,7 +781,7 @@ func QueryArrayEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 32
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"instock.0.qty", bson.D{
 					{"$lte", 20},
@@ -798,7 +798,7 @@ func QueryArrayEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 33
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"instock.qty", bson.D{
 					{"$lte", 20},
@@ -815,7 +815,7 @@ func QueryArrayEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 34
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"instock", bson.D{
 					{"$elemMatch", bson.D{
@@ -835,7 +835,7 @@ func QueryArrayEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 35
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"instock", bson.D{
 					{"$elemMatch", bson.D{
@@ -857,7 +857,7 @@ func QueryArrayEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 36
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"instock.qty", bson.D{
 					{"$gt", 10},
@@ -875,7 +875,7 @@ func QueryArrayEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 37
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"instock.qty", 5},
 				{"instock.warehouse", "A"},
@@ -889,6 +889,7 @@ func QueryArrayEmbeddedDocumentsExamples(t *testing.T, db *mongo.Database) {
 }
 
 // QueryNullMissingFieldsExamples contains examples for querying fields that are null or missing.
+// Appears at https://www.mongodb.com/docs/manual/tutorial/query-for-null-fields/.
 func QueryNullMissingFieldsExamples(t *testing.T, db *mongo.Database) {
 	coll := db.Collection("inventory_query_null_missing")
 
@@ -908,7 +909,7 @@ func QueryNullMissingFieldsExamples(t *testing.T, db *mongo.Database) {
 			},
 		}
 
-		result, err := coll.InsertMany(context.Background(), docs)
+		result, err := coll.InsertMany(context.TODO(), docs)
 
 		// End Example 38
 
@@ -920,7 +921,7 @@ func QueryNullMissingFieldsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 39
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"item", nil},
 			})
@@ -935,7 +936,7 @@ func QueryNullMissingFieldsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 40
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"item", bson.D{
 					{"$type", 10},
@@ -952,7 +953,7 @@ func QueryNullMissingFieldsExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 41
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"item", bson.D{
 					{"$exists", false},
@@ -967,6 +968,7 @@ func QueryNullMissingFieldsExamples(t *testing.T, db *mongo.Database) {
 }
 
 // ProjectionExamples contains examples for specifying projections in find operations.
+// Appears at https://www.mongodb.com/docs/manual/tutorial/project-fields-from-query-results/.
 func ProjectionExamples(t *testing.T, db *mongo.Database) {
 	coll := db.Collection("inventory_project")
 
@@ -1058,7 +1060,7 @@ func ProjectionExamples(t *testing.T, db *mongo.Database) {
 			},
 		}
 
-		result, err := coll.InsertMany(context.Background(), docs)
+		result, err := coll.InsertMany(context.TODO(), docs)
 
 		// End Example 42
 
@@ -1070,7 +1072,7 @@ func ProjectionExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 43
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{{"status", "A"}},
 		)
 
@@ -1089,7 +1091,7 @@ func ProjectionExamples(t *testing.T, db *mongo.Database) {
 		}
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"status", "A"},
 			},
@@ -1123,7 +1125,7 @@ func ProjectionExamples(t *testing.T, db *mongo.Database) {
 		}
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"status", "A"},
 			},
@@ -1156,7 +1158,7 @@ func ProjectionExamples(t *testing.T, db *mongo.Database) {
 		}
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"status", "A"},
 			},
@@ -1190,7 +1192,7 @@ func ProjectionExamples(t *testing.T, db *mongo.Database) {
 		}
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"status", "A"},
 			},
@@ -1227,7 +1229,7 @@ func ProjectionExamples(t *testing.T, db *mongo.Database) {
 		}
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"status", "A"},
 			},
@@ -1266,7 +1268,7 @@ func ProjectionExamples(t *testing.T, db *mongo.Database) {
 		}
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"status", "A"},
 			},
@@ -1319,7 +1321,7 @@ func ProjectionExamples(t *testing.T, db *mongo.Database) {
 		}
 
 		cursor, err := coll.Find(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"status", "A"},
 			},
@@ -1351,6 +1353,7 @@ func ProjectionExamples(t *testing.T, db *mongo.Database) {
 }
 
 // UpdateExamples contains examples of update operations.
+// Appears at https://www.mongodb.com/docs/manual/tutorial/update-documents/.
 func UpdateExamples(t *testing.T, db *mongo.Database) {
 	coll := db.Collection("inventory_update")
 
@@ -1463,7 +1466,7 @@ func UpdateExamples(t *testing.T, db *mongo.Database) {
 			},
 		}
 
-		result, err := coll.InsertMany(context.Background(), docs)
+		result, err := coll.InsertMany(context.TODO(), docs)
 
 		// End Example 51
 
@@ -1475,7 +1478,7 @@ func UpdateExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 52
 
 		result, err := coll.UpdateOne(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"item", "paper"},
 			},
@@ -1525,7 +1528,7 @@ func UpdateExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 53
 
 		result, err := coll.UpdateMany(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"qty", bson.D{
 					{"$lt", 50},
@@ -1579,7 +1582,7 @@ func UpdateExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 54
 
 		result, err := coll.ReplaceOne(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"item", "paper"},
 			},
@@ -1631,6 +1634,7 @@ func UpdateExamples(t *testing.T, db *mongo.Database) {
 }
 
 // DeleteExamples contains examples of delete operations.
+// Appears at https://www.mongodb.com/docs/manual/tutorial/remove-documents/.
 func DeleteExamples(t *testing.T, db *mongo.Database) {
 	coll := db.Collection("inventory_delete")
 
@@ -1692,7 +1696,7 @@ func DeleteExamples(t *testing.T, db *mongo.Database) {
 			},
 		}
 
-		result, err := coll.InsertMany(context.Background(), docs)
+		result, err := coll.InsertMany(context.TODO(), docs)
 
 		// End Example 55
 
@@ -1704,7 +1708,7 @@ func DeleteExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 57
 
 		result, err := coll.DeleteMany(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"status", "A"},
 			},
@@ -1720,7 +1724,7 @@ func DeleteExamples(t *testing.T, db *mongo.Database) {
 		// Start Example 58
 
 		result, err := coll.DeleteOne(
-			context.Background(),
+			context.TODO(),
 			bson.D{
 				{"status", "D"},
 			},
@@ -1736,7 +1740,7 @@ func DeleteExamples(t *testing.T, db *mongo.Database) {
 	{
 		// Start Example 56
 
-		result, err := coll.DeleteMany(context.Background(), bson.D{})
+		result, err := coll.DeleteMany(context.TODO(), bson.D{})
 
 		// End Example 56
 
@@ -1747,6 +1751,8 @@ func DeleteExamples(t *testing.T, db *mongo.Database) {
 
 var log = logger.New(ioutil.Discard, "", logger.LstdFlags)
 
+// Transactions examples below all appear at https://www.mongodb.com/docs/manual/core/transactions-in-applications/.
+
 // Start Transactions Intro Example 1
 
 // UpdateEmployeeInfo is an example function demonstrating transactions.
@@ -1950,18 +1956,17 @@ func TransactionsExamples(ctx context.Context, client *mongo.Client) error {
 // Start Transactions withTxn API Example 1
 
 // WithTransactionExample is an example of using the Session.WithTransaction function.
-func WithTransactionExample() {
-	ctx := context.Background()
+func WithTransactionExample(ctx context.Context) error {
 	// For a replica set, include the replica set name and a seedlist of the members in the URI string; e.g.
 	// uri := "mongodb://mongodb0.example.com:27017,mongodb1.example.com:27017/?replicaSet=myRepl"
 	// For a sharded cluster, connect to the mongos instances; e.g.
 	// uri := "mongodb://mongos0.example.com:27017,mongos1.example.com:27017/"
-	var uri string
+	uri := mtest.ClusterURI()
 
 	clientOpts := options.Client().ApplyURI(uri)
 	client, err := mongo.Connect(ctx, clientOpts)
 	if err != nil {
-		panic(err)
+		return err
 	}
 	defer func() { _ = client.Disconnect(ctx) }()
 
@@ -1988,20 +1993,22 @@ func WithTransactionExample() {
 	// Step 2: Start a session and run the callback using WithTransaction.
 	session, err := client.StartSession()
 	if err != nil {
-		panic(err)
+		return err
 	}
 	defer session.EndSession(ctx)
 
 	result, err := session.WithTransaction(ctx, callback)
 	if err != nil {
-		panic(err)
+		return err
 	}
-	fmt.Printf("result: %v\n", result)
+	log.Printf("result: %v\n", result)
+	return nil
 }
 
 // End Transactions withTxn API Example 1
 
 // ChangeStreamExamples contains examples of changestream operations.
+// Appears at https://www.mongodb.com/docs/manual/changeStreams/.
 func ChangeStreamExamples(t *testing.T, db *mongo.Database) {
 	ctx := context.Background()
 
@@ -2108,6 +2115,7 @@ func ChangeStreamExamples(t *testing.T, db *mongo.Database) {
 }
 
 // AggregationExamples contains examples of aggregation operations.
+// Appears at https://www.mongodb.com/docs/manual/aggregation/.
 func AggregationExamples(t *testing.T, db *mongo.Database) {
 	ctx := context.Background()
 
@@ -2528,6 +2536,7 @@ func AggregationExamples(t *testing.T, db *mongo.Database) {
 }
 
 // CausalConsistencyExamples contains examples of causal consistency usage.
+// Appears at https://www.mongodb.com/docs/manual/core/read-isolation-consistency-recency/.
 func CausalConsistencyExamples(client *mongo.Client) error {
 	ctx := context.Background()
 	coll := client.Database("test").Collection("items")
@@ -2545,10 +2554,10 @@ func CausalConsistencyExamples(client *mongo.Client) error {
 	opts := options.Session().SetDefaultReadConcern(readconcern.Majority()).SetDefaultWriteConcern(
 		writeconcern.New(writeconcern.WMajority(), writeconcern.WTimeout(1000)))
 	session1, err := client.StartSession(opts)
-
 	if err != nil {
 		return err
 	}
+	defer session1.EndSession(context.TODO())
 
 	err = client.UseSessionWithOptions(context.TODO(), opts, func(sctx mongo.SessionContext) error {
 		// Run an update with our causally-consistent session
@@ -2579,10 +2588,10 @@ func CausalConsistencyExamples(client *mongo.Client) error {
 		readconcern.Majority()).SetDefaultWriteConcern(writeconcern.New(writeconcern.WMajority(),
 		writeconcern.WTimeout(1000)))
 	session2, err := client.StartSession(opts)
-
 	if err != nil {
 		return err
 	}
+	defer session2.EndSession(context.TODO())
 
 	err = client.UseSessionWithOptions(context.TODO(), opts, func(sctx mongo.SessionContext) error {
 		// Set cluster time of session2 to session1's cluster time
@@ -2616,6 +2625,7 @@ func CausalConsistencyExamples(client *mongo.Client) error {
 }
 
 // RunCommandExamples contains examples of RunCommand operations.
+// Appears at https://www.mongodb.com/docs/manual/reference/command/buildInfo/.
 func RunCommandExamples(t *testing.T, db *mongo.Database) {
 	ctx := context.Background()
 
@@ -2624,67 +2634,19 @@ func RunCommandExamples(t *testing.T, db *mongo.Database) {
 	err := coll.Drop(ctx)
 	require.NoError(t, err)
 
-	restaurants := []interface{}{
-		bson.D{
-			{"name", "Chez Panisse"},
-			{"city", "Oakland"},
-			{"state", "California"},
-			{"country", "United States"},
-			{"rating", 4.4},
-		},
-		bson.D{
-			{"name", "Central"},
-			{"city", "Lima"},
-			{"country", "Peru"},
-			{"rating", 4.8},
-		},
-		bson.D{
-			{"name", "Eleven Madison Park"},
-			{"city", "New York City"},
-			{"state", "New York"},
-			{"country", "United States"},
-			{"rating", 4.6},
-		},
-		bson.D{
-			{"name", "Gaggan"},
-			{"city", "Bangkok"},
-			{"country", "Thailand"},
-			{"rating", 4.3},
-		},
-		bson.D{
-			{"name", "Dad's Grill"},
-			{"city", "Oklahoma City"},
-			{"state", "Oklahoma"},
-			{"country", "United States"},
-			{"rating", 2.1},
-		},
-	}
-
-	result, err := coll.InsertMany(ctx, restaurants)
-	require.NoError(t, err)
-	require.Len(t, result.InsertedIDs, 5)
-
 	{
 		// Start RunCommand Example 1
 		res := db.RunCommand(ctx, bson.D{{"buildInfo", 1}})
 
 		// End RunCommand Example 1
 
-		err := res.Err()
-		require.NoError(t, err)
-	}
-	{
-		// Start RunCommand Example 2
-		res := db.RunCommand(ctx, bson.D{{"collStats", "restaurants"}})
-
-		// End RunCommand Example 2
-
 		err := res.Err()
 		require.NoError(t, err)
 	}
 }
 
 // IndexExamples contains examples of Index operations.
+// Appears at https://www.mongodb.com/docs/manual/indexes/.
 func IndexExamples(t *testing.T, db *mongo.Database) {
 	ctx := context.Background()
 
@@ -2804,14 +2766,14 @@ func IndexExamples(t *testing.T, db *mongo.Database) {
 
 // Start Versioned API Example 1
 
-// VersionedAPIExample is an example of creating a client with versioned API.
-func VersionedAPIExample() {
-	ctx := context.Background()
+// StableAPIExample is an example of creating a client with stable API.
+func StableAPIExample() {
+	ctx := context.TODO()
 	// For a replica set, include the replica set name and a seedlist of the members in the URI string; e.g.
 	// uri := "mongodb://mongodb0.example.com:27017,mongodb1.example.com:27017/?replicaSet=myRepl"
 	// For a sharded cluster, connect to the mongos instances; e.g.
 	// uri := "mongodb://mongos0.example.com:27017,mongos1.example.com:27017/"
-	uri := "mongodb://localhost:27017"
+	uri := mtest.ClusterURI()
 
 	serverAPIOptions := options.ServerAPI(options.ServerAPIVersion1)
 	clientOpts := options.Client().ApplyURI(uri).SetServerAPIOptions(serverAPIOptions)
@@ -2826,14 +2788,14 @@ func VersionedAPIExample() {
 
 // Start Versioned API Example 2
 
-// VersionedAPIStrictExample is an example of creating a client with strict versioned API.
-func VersionedAPIStrictExample() {
-	ctx := context.Background()
+// StableAPIStrictExample is an example of creating a client with strict stable API.
+func StableAPIStrictExample() {
+	ctx := context.TODO()
 	// For a replica set, include the replica set name and a seedlist of the members in the URI string; e.g.
 	// uri := "mongodb://mongodb0.example.com:27017,mongodb1.example.com:27017/?replicaSet=myRepl"
 	// For a sharded cluster, connect to the mongos instances; e.g.
 	// uri := "mongodb://mongos0.example.com:27017,mongos1.example.com:27017/"
-	uri := "mongodb://localhost:27017"
+	uri := mtest.ClusterURI()
 
 	serverAPIOptions := options.ServerAPI(options.ServerAPIVersion1).SetStrict(true)
 	clientOpts := options.Client().ApplyURI(uri).SetServerAPIOptions(serverAPIOptions)
@@ -2848,14 +2810,14 @@ func VersionedAPIStrictExample() {
 
 // Start Versioned API Example 3
 
-// VersionedAPINonStrictExample is an example of creating a client with non-strict versioned API.
-func VersionedAPINonStrictExample() {
-	ctx := context.Background()
+// StableAPINonStrictExample is an example of creating a client with non-strict stable API.
+func StableAPINonStrictExample() {
+	ctx := context.TODO()
 	// For a replica set, include the replica set name and a seedlist of the members in the URI string; e.g.
 	// uri := "mongodb://mongodb0.example.com:27017,mongodb1.example.com:27017/?replicaSet=myRepl"
 	// For a sharded cluster, connect to the mongos instances; e.g.
 	// uri := "mongodb://mongos0.example.com:27017,mongos1.example.com:27017/"
-	uri := "mongodb://localhost:27017"
+	uri := mtest.ClusterURI()
 
 	serverAPIOptions := options.ServerAPI(options.ServerAPIVersion1).SetStrict(false)
 	clientOpts := options.Client().ApplyURI(uri).SetServerAPIOptions(serverAPIOptions)
@@ -2870,15 +2832,15 @@ func VersionedAPINonStrictExample() {
 
 // Start Versioned API Example 4
 
-// VersionedAPIDeprecationErrorsExample is an example of creating a client with versioned API
+// StableAPIDeprecationErrorsExample is an example of creating a client with stable API
 // with deprecation errors.
-func VersionedAPIDeprecationErrorsExample() {
-	ctx := context.Background()
+func StableAPIDeprecationErrorsExample() {
+	ctx := context.TODO()
 	// For a replica set, include the replica set name and a seedlist of the members in the URI string; e.g.
 	// uri := "mongodb://mongodb0.example.com:27017,mongodb1.example.com:27017/?replicaSet=myRepl"
 	// For a sharded cluster, connect to the mongos instances; e.g.
 	// uri := "mongodb://mongos0.example.com:27017,mongos1.example.com:27017/"
-	uri := "mongodb://localhost:27017"
+	uri := mtest.ClusterURI()
 
 	serverAPIOptions := options.ServerAPI(options.ServerAPIVersion1).SetDeprecationErrors(true)
 	clientOpts := options.Client().ApplyURI(uri).SetServerAPIOptions(serverAPIOptions)
@@ -2891,10 +2853,14 @@ func VersionedAPIDeprecationErrorsExample() {
 
 // End Versioned API Example 4
 
-// VersionedAPIStrictCountExample is an example of using CountDocuments instead of a traditional count
-// with a strict API version since the count command does not belong to API version 1.
-func VersionedAPIStrictCountExample(t *testing.T) {
-	uri := "mongodb://localhost:27017"
+// StableAPIStrictCountExample is an example of using CountDocuments instead of a traditional count
+// with a strict stable API since the count command does not belong to API version 1.
+func StableAPIStrictCountExample(t *testing.T) {
+	// TODO(GODRIVER-2482): The "count" command is now part of Stable API v1 in MongoDB v5.0.x and
+	// TODO v6.x, so this example no longer works correctly in any CI tested server version. Rewrite
+	// TODO this example with a command that is not part of Stable API v1. For now, this example
+	// TODO test is always skipped.
+	uri := mtest.ClusterURI()
 
 	serverAPIOptions := options.ServerAPI(options.ServerAPIVersion1).SetStrict(true)
 	clientOpts := options.Client().ApplyURI(uri).SetServerAPIOptions(serverAPIOptions)
@@ -2949,10 +2915,188 @@ func VersionedAPIStrictCountExample(t *testing.T) {
 	// End Versioned API Example 8
 }
 
-// VersionedAPIExamples runs all versioned API examples.
-func VersionedAPIExamples() {
-	VersionedAPIExample()
-	VersionedAPIStrictExample()
-	VersionedAPINonStrictExample()
-	VersionedAPIDeprecationErrorsExample()
+// StableAPIExamples runs all stable API examples.
+// These appear at https://www.mongodb.com/docs/manual/reference/stable-api/.
+func StableAPIExamples() {
+	StableAPIExample()
+	StableAPIStrictExample()
+	StableAPINonStrictExample()
+	StableAPIDeprecationErrorsExample()
+}
+
+func insertSnapshotQueryTestData(mt *mtest.T) {
+	catColl := mt.CreateCollection(mtest.Collection{Name: "cats"}, true)
+	_, err := catColl.InsertMany(context.Background(), []interface{}{
+		bson.D{
+			{"adoptable", false},
+			{"name", "Miyagi"},
+			{"color", "grey-white"},
+			{"age", 14},
+		},
+		bson.D{
+			{"adoptable", true},
+			{"name", "Joyce"},
+			{"color", "black"},
+			{"age", 10},
+		},
+	})
+	require.NoError(mt, err)
+
+	dogColl := mt.CreateCollection(mtest.Collection{Name: "dogs"}, true)
+	_, err = dogColl.InsertMany(context.Background(), []interface{}{
+		bson.D{
+			{"adoptable", true},
+			{"name", "Cormac"},
+			{"color", "rust"},
+			{"age", 7},
+		},
+		bson.D{
+			{"adoptable", true},
+			{"name", "Frank"},
+			{"color", "yellow"},
+			{"age", 2},
+		},
+	})
+	require.NoError(mt, err)
+
+	salesColl := mt.CreateCollection(mtest.Collection{Name: "sales"}, true)
+	_, err = salesColl.InsertMany(context.Background(), []interface{}{
+		bson.D{
+			{"shoeType", "hiking boot"},
+			{"price", 30.0},
+			{"saleDate", time.Now()},
+		},
+	})
+	require.NoError(mt, err)
+}
+
+func snapshotQueryPetExample(mt *mtest.T) error {
+	client := mt.Client
+	db := mt.DB
+
+	// Start Snapshot Query Example 1
+	ctx := context.TODO()
+
+	sess, err := client.StartSession(options.Session().SetSnapshot(true))
+	if err != nil {
+		return err
+	}
+	defer sess.EndSession(ctx)
+
+	var adoptablePetsCount int32
+	err = mongo.WithSession(ctx, sess, func(ctx mongo.SessionContext) error {
+		// Count the adoptable cats
+		const adoptableCatsOutput = "adoptableCatsCount"
+		cursor, err := db.Collection("cats").Aggregate(ctx, mongo.Pipeline{
+			bson.D{{"$match", bson.D{{"adoptable", true}}}},
+			bson.D{{"$count", adoptableCatsOutput}},
+		})
+		if err != nil {
+			return err
+		}
+		if !cursor.Next(ctx) {
+			return fmt.Errorf("expected aggregate to return a document, but got none")
+		}
+
+		resp := cursor.Current.Lookup(adoptableCatsOutput)
+		adoptableCatsCount, ok := resp.Int32OK()
+		if !ok {
+			return fmt.Errorf("failed to find int32 field %q in document %v", adoptableCatsOutput, cursor.Current)
+		}
+		adoptablePetsCount += adoptableCatsCount
+
+		// Count the adoptable dogs
+		const adoptableDogsOutput = "adoptableDogsCount"
+		cursor, err = db.Collection("dogs").Aggregate(ctx, mongo.Pipeline{
+			bson.D{{"$match", bson.D{{"adoptable", true}}}},
+			bson.D{{"$count", adoptableDogsOutput}},
+		})
+		if err != nil {
+			return err
+		}
+		if !cursor.Next(ctx) {
+			return fmt.Errorf("expected aggregate to return a document, but got none")
+		}
+
+		resp = cursor.Current.Lookup(adoptableDogsOutput)
+		adoptableDogsCount, ok := resp.Int32OK()
+		if !ok {
+			return fmt.Errorf("failed to find int32 field %q in document %v", adoptableDogsOutput, cursor.Current)
+		}
+		adoptablePetsCount += adoptableDogsCount
+		return nil
+	})
+	if err != nil {
+		return err
+	}
+	// End Snapshot Query Example 1
+	require.Equal(mt, int32(3), adoptablePetsCount, "expected 3 total adoptable pets")
+	return nil
+}
+
+func snapshotQueryRetailExample(mt *mtest.T) error {
+	client := mt.Client
+	db := mt.DB
+
+	// Start Snapshot Query Example 2
+	ctx := context.TODO()
+
+	sess, err := client.StartSession(options.Session().SetSnapshot(true))
+	if err != nil {
+		return err
+	}
+	defer sess.EndSession(ctx)
+
+	var totalDailySales int32
+	err = mongo.WithSession(ctx, sess, func(ctx mongo.SessionContext) error {
+		// Count the total daily sales
+		const totalDailySalesOutput = "totalDailySales"
+		cursor, err := db.Collection("sales").Aggregate(ctx, mongo.Pipeline{
+			bson.D{{"$match",
+				bson.D{{"$expr",
+					bson.D{{"$gt",
+						bson.A{"$saleDate",
+							bson.D{{"$dateSubtract",
+								bson.D{
+									{"startDate", "$$NOW"},
+									{"unit", "day"},
+									{"amount", 1},
+								},
+							}},
+						},
+					}},
+				}},
+			}},
+			bson.D{{"$count", totalDailySalesOutput}},
+		})
+		if err != nil {
+			return err
+		}
+		if !cursor.Next(ctx) {
+			return fmt.Errorf("expected aggregate to return a document, but got none")
+		}
+
+		resp := cursor.Current.Lookup(totalDailySalesOutput)
+
+		var ok bool
+		totalDailySales, ok = resp.Int32OK()
+		if !ok {
+			return fmt.Errorf("failed to find int32 field %q in document %v", totalDailySalesOutput, cursor.Current)
+		}
+		return nil
+	})
+	if err != nil {
+		return err
+	}
+	// End Snapshot Query Example 2
+	require.Equal(mt, int32(1), totalDailySales, "expected 1 total daily sale")
+	return nil
+}
+
+// SnapshotQuery examples runs examples of using sessions with Snapshot enabled.
+// These appear at https://www.mongodb.com/docs/manual/tutorial/long-running-queries/.
+func SnapshotQueryExamples(mt *mtest.T) {
+	insertSnapshotQueryTestData(mt)
+	require.NoError(mt, snapshotQueryPetExample(mt))
+	require.NoError(mt, snapshotQueryRetailExample(mt))
 }
