@@ -22,14 +22,18 @@ import (
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/event"
 	"go.mongodb.org/mongo-driver/internal"
+	"go.mongodb.org/mongo-driver/internal/assert"
+	"go.mongodb.org/mongo-driver/internal/require"
 	"go.mongodb.org/mongo-driver/internal/testutil"
-	"go.mongodb.org/mongo-driver/internal/testutil/assert"
+	"go.mongodb.org/mongo-driver/internal/testutil/helpers"
+	"go.mongodb.org/mongo-driver/internal/testutil/monitor"
 	"go.mongodb.org/mongo-driver/mongo"
 	"go.mongodb.org/mongo-driver/mongo/integration/mtest"
 	"go.mongodb.org/mongo-driver/mongo/options"
 	"go.mongodb.org/mongo-driver/mongo/readpref"
 	"go.mongodb.org/mongo-driver/x/bsonx/bsoncore"
 	"go.mongodb.org/mongo-driver/x/mongo/driver"
+	"go.mongodb.org/mongo-driver/x/mongo/driver/wiremessage"
 	"golang.org/x/sync/errgroup"
 )
 
@@ -39,12 +43,12 @@ type negateCodec struct {
 	ID int64 `bson:"_id"`
 }
 
-func (e *negateCodec) EncodeValue(ectx bsoncodec.EncodeContext, vw bsonrw.ValueWriter, val reflect.Value) error {
+func (e *negateCodec) EncodeValue(_ bsoncodec.EncodeContext, vw bsonrw.ValueWriter, val reflect.Value) error {
 	return vw.WriteInt64(val.Int())
 }
 
 // DecodeValue negates the value of ID when reading
-func (e *negateCodec) DecodeValue(ectx bsoncodec.DecodeContext, vr bsonrw.ValueReader, val reflect.Value) error {
+func (e *negateCodec) DecodeValue(_ bsoncodec.DecodeContext, vr bsonrw.ValueReader, val reflect.Value) error {
 	i, err := vr.ReadInt64()
 	if err != nil {
 		return err
@@ -62,6 +66,9 @@ type slowConnDialer struct {
 	delay  time.Duration
 }
 
+var slowConnDialerDelay = 300 * time.Millisecond
+var reducedHeartbeatInterval = 100 * time.Millisecond
+
 func newSlowConnDialer(delay time.Duration) *slowConnDialer {
 	return &slowConnDialer{
 		dialer: &net.Dialer{},
@@ -101,17 +108,17 @@ func TestClient(t *testing.T) {
 	registryOpts := options.Client().
 		SetRegistry(bson.NewRegistryBuilder().RegisterCodec(reflect.TypeOf(int64(0)), &negateCodec{}).Build())
 	mt.RunOpts("registry passed to cursors", mtest.NewOptions().ClientOptions(registryOpts), func(mt *mtest.T) {
-		_, err := mt.Coll.InsertOne(mtest.Background, negateCodec{ID: 10})
+		_, err := mt.Coll.InsertOne(context.Background(), negateCodec{ID: 10})
 		assert.Nil(mt, err, "InsertOne error: %v", err)
 		var got negateCodec
-		err = mt.Coll.FindOne(mtest.Background, bson.D{}).Decode(&got)
+		err = mt.Coll.FindOne(context.Background(), bson.D{}).Decode(&got)
 		assert.Nil(mt, err, "Find error: %v", err)
 
 		assert.Equal(mt, int64(-10), got.ID, "expected ID -10, got %v", got.ID)
 	})
 	mt.RunOpts("tls connection", mtest.NewOptions().MinServerVersion("3.0").SSL(true), func(mt *mtest.T) {
 		var result bson.Raw
-		err := mt.Coll.Database().RunCommand(mtest.Background, bson.D{
+		err := mt.Coll.Database().RunCommand(context.Background(), bson.D{
 			{"serverStatus", 1},
 		}).Decode(&result)
 		assert.Nil(mt, err, "serverStatus error: %v", err)
@@ -149,11 +156,11 @@ func TestClient(t *testing.T) {
 
 				// We don't care if the user doesn't already exist.
 				_ = db.RunCommand(
-					mtest.Background,
+					context.Background(),
 					bson.D{{"dropUser", user}},
 				)
 				err := db.RunCommand(
-					mtest.Background,
+					context.Background(),
 					bson.D{
 						{"createUser", user},
 						{"roles", bson.A{
@@ -178,11 +185,11 @@ func TestClient(t *testing.T) {
 				)
 				authClientOpts := options.Client().ApplyURI(cs)
 				testutil.AddTestServerAPIVersion(authClientOpts)
-				authClient, err := mongo.Connect(mtest.Background, authClientOpts)
+				authClient, err := mongo.Connect(context.Background(), authClientOpts)
 				assert.Nil(mt, err, "authClient Connect error: %v", err)
-				defer func() { _ = authClient.Disconnect(mtest.Background) }()
+				defer func() { _ = authClient.Disconnect(context.Background()) }()
 
-				rdr, err := authClient.Database("test").RunCommand(mtest.Background, bson.D{
+				rdr, err := authClient.Database("test").RunCommand(context.Background(), bson.D{
 					{"connectionStatus", 1},
 				}).DecodeBytes()
 				assert.Nil(mt, err, "connectionStatus error: %v", err)
@@ -228,7 +235,7 @@ func TestClient(t *testing.T) {
 				}
 
 				mt.RunOpts(tc.name, opts, func(mt *mtest.T) {
-					res, err := mt.Client.ListDatabases(mtest.Background, tc.filter)
+					res, err := mt.Client.ListDatabases(context.Background(), tc.filter)
 					assert.Nil(mt, err, "ListDatabases error: %v", err)
 
 					var found bool
@@ -246,7 +253,7 @@ func TestClient(t *testing.T) {
 			allOpts := options.ListDatabases().SetNameOnly(true).SetAuthorizedDatabases(true)
 			mt.ClearEvents()
 
-			_, err := mt.Client.ListDatabases(mtest.Background, bson.D{}, allOpts)
+			_, err := mt.Client.ListDatabases(context.Background(), bson.D{}, allOpts)
 			assert.Nil(mt, err, "ListDatabases error: %v", err)
 
 			evt := mt.GetStartedEvent()
@@ -279,7 +286,7 @@ func TestClient(t *testing.T) {
 				}
 
 				mt.RunOpts(tc.name, opts, func(mt *mtest.T) {
-					dbs, err := mt.Client.ListDatabaseNames(mtest.Background, tc.filter)
+					dbs, err := mt.Client.ListDatabaseNames(context.Background(), tc.filter)
 					assert.Nil(mt, err, "ListDatabaseNames error: %v", err)
 
 					var found bool
@@ -297,7 +304,7 @@ func TestClient(t *testing.T) {
 			allOpts := options.ListDatabases().SetNameOnly(true).SetAuthorizedDatabases(true)
 			mt.ClearEvents()
 
-			_, err := mt.Client.ListDatabaseNames(mtest.Background, bson.D{}, allOpts)
+			_, err := mt.Client.ListDatabaseNames(context.Background(), bson.D{}, allOpts)
 			assert.Nil(mt, err, "ListDatabaseNames error: %v", err)
 
 			evt := mt.GetStartedEvent()
@@ -313,7 +320,7 @@ func TestClient(t *testing.T) {
 	})
 	mt.RunOpts("ping", noClientOpts, func(mt *mtest.T) {
 		mt.Run("default read preference", func(mt *mtest.T) {
-			err := mt.Client.Ping(mtest.Background, nil)
+			err := mt.Client.Ping(context.Background(), nil)
 			assert.Nil(mt, err, "Ping error: %v", err)
 		})
 		mt.Run("invalid host", func(mt *mtest.T) {
@@ -323,11 +330,11 @@ func TestClient(t *testing.T) {
 				SetServerSelectionTimeout(100 * time.Millisecond).SetHosts([]string{"invalid:123"}).
 				SetConnectTimeout(500 * time.Millisecond).SetSocketTimeout(500 * time.Millisecond)
 			testutil.AddTestServerAPIVersion(invalidClientOpts)
-			client, err := mongo.Connect(mtest.Background, invalidClientOpts)
+			client, err := mongo.Connect(context.Background(), invalidClientOpts)
 			assert.Nil(mt, err, "Connect error: %v", err)
-			err = client.Ping(mtest.Background, readpref.Primary())
+			err = client.Ping(context.Background(), readpref.Primary())
 			assert.NotNil(mt, err, "expected error for pinging invalid host, got nil")
-			_ = client.Disconnect(mtest.Background)
+			_ = client.Disconnect(context.Background())
 		})
 	})
 	mt.RunOpts("disconnect", noClientOpts, func(mt *mtest.T) {
@@ -340,23 +347,23 @@ func TestClient(t *testing.T) {
 		mt.Run("disconnected", func(mt *mtest.T) {
 			c, err := mongo.NewClient(options.Client().ApplyURI(mtest.ClusterURI()))
 			assert.Nil(mt, err, "NewClient error: %v", err)
-			_, err = c.Watch(mtest.Background, mongo.Pipeline{})
+			_, err = c.Watch(context.Background(), mongo.Pipeline{})
 			assert.Equal(mt, mongo.ErrClientDisconnected, err, "expected error %v, got %v", mongo.ErrClientDisconnected, err)
 		})
 	})
 	mt.RunOpts("end sessions", mtest.NewOptions().MinServerVersion("3.6"), func(mt *mtest.T) {
-		_, err := mt.Client.ListDatabases(mtest.Background, bson.D{})
+		_, err := mt.Client.ListDatabases(context.Background(), bson.D{})
 		assert.Nil(mt, err, "ListDatabases error: %v", err)
 
 		mt.ClearEvents()
-		err = mt.Client.Disconnect(mtest.Background)
+		err = mt.Client.Disconnect(context.Background())
 		assert.Nil(mt, err, "Disconnect error: %v", err)
 
 		started := mt.GetStartedEvent()
 		assert.Equal(mt, "endSessions", started.CommandName, "expected cmd name endSessions, got %v", started.CommandName)
 	})
 	mt.RunOpts("hello lastWriteDate", mtest.NewOptions().Topologies(mtest.ReplicaSet), func(mt *mtest.T) {
-		_, err := mt.Coll.InsertOne(mtest.Background, bson.D{{"x", 1}})
+		_, err := mt.Coll.InsertOne(context.Background(), bson.D{{"x", 1}})
 		assert.Nil(mt, err, "InsertOne error: %v", err)
 	})
 	sessionOpts := mtest.NewOptions().MinServerVersion("3.6.0").CreateClient(false)
@@ -375,7 +382,7 @@ func TestClient(t *testing.T) {
 			mt.Run(tc.name, func(mt *mtest.T) {
 				sess, err := mt.Client.StartSession(tc.opts)
 				assert.Nil(mt, err, "StartSession error: %v", err)
-				defer sess.EndSession(mtest.Background)
+				defer sess.EndSession(context.Background())
 				xs := sess.(mongo.XSession)
 				consistent := xs.ClientSession().Consistent
 				assert.Equal(mt, tc.consistent, consistent, "expected consistent to be %v, got %v", tc.consistent, consistent)
@@ -428,9 +435,9 @@ func TestClient(t *testing.T) {
 
 				sess, err := mt.Client.StartSession()
 				assert.Nil(mt, err, "StartSession error: %v", err)
-				defer sess.EndSession(mtest.Background)
+				defer sess.EndSession(context.Background())
 
-				_, err = mt.Coll.InsertOne(mtest.Background, bson.D{{"x", 1}})
+				_, err = mt.Coll.InsertOne(context.Background(), bson.D{{"x", 1}})
 				assert.NotNil(mt, err, "expected err but got nil")
 				if tc.expectUnsupportedMsg {
 					assert.Equal(mt, driver.ErrUnsupportedStorageEngine.Error(), err.Error(),
@@ -451,7 +458,7 @@ func TestClient(t *testing.T) {
 		ClientOptions(appNameClientOpts).
 		Topologies(mtest.Single)
 	mt.RunOpts("app name is always sent", appNameMtOpts, func(mt *mtest.T) {
-		err := mt.Client.Ping(mtest.Background, mtest.PrimaryRp)
+		err := mt.Client.Ping(context.Background(), mtest.PrimaryRp)
 		assert.Nil(mt, err, "Ping error: %v", err)
 
 		msgPairs := mt.GetProxiedMessages()
@@ -489,7 +496,7 @@ func TestClient(t *testing.T) {
 		MinServerVersion("3.6").     // Minimum server version 3.6 to force OP_MSG.
 		Topologies(mtest.ReplicaSet) // Read preference isn't sent to standalones so we can test on replica sets.
 	mt.RunOpts("direct connection made", mtOpts, func(mt *mtest.T) {
-		_, err := mt.Coll.Find(mtest.Background, bson.D{})
+		_, err := mt.Coll.Find(context.Background(), bson.D{})
 		assert.Nil(mt, err, "Find error: %v", err)
 
 		// When connected directly, the primary read preference should be overwritten to primaryPreferred.
@@ -510,20 +517,25 @@ func TestClient(t *testing.T) {
 	})
 
 	mt.Run("minimum RTT is monitored", func(mt *mtest.T) {
-		if testing.Short() {
-			t.Skip("skipping integration test in short mode")
-		}
+		mt.Parallel()
 
 		// Reset the client with a dialer that delays all network round trips by 300ms and set the
 		// heartbeat interval to 100ms to reduce the time it takes to collect RTT samples.
 		mt.ResetClient(options.Client().
-			SetDialer(newSlowConnDialer(300 * time.Millisecond)).
-			SetHeartbeatInterval(100 * time.Millisecond))
+			SetDialer(newSlowConnDialer(slowConnDialerDelay)).
+			SetHeartbeatInterval(reducedHeartbeatInterval))
 
 		// Assert that the minimum RTT is eventually >250ms.
 		topo := getTopologyFromClient(mt.Client)
-		assert.Soon(mt, func() {
+		helpers.AssertSoon(mt, func(ctx context.Context) {
 			for {
+				// Stop loop if callback has been canceled.
+				select {
+				case <-ctx.Done():
+					return
+				default:
+				}
+
 				time.Sleep(100 * time.Millisecond)
 
 				// Wait for all of the server's minimum RTTs to be >250ms.
@@ -531,7 +543,7 @@ func TestClient(t *testing.T) {
 				for _, desc := range topo.Description().Servers {
 					server, err := topo.FindServer(desc)
 					assert.Nil(mt, err, "FindServer error: %v", err)
-					if server.MinRTT() <= 250*time.Millisecond {
+					if server.RTTMonitor().Min() <= 250*time.Millisecond {
 						done = false
 					}
 				}
@@ -545,9 +557,7 @@ func TestClient(t *testing.T) {
 	// Test that if the minimum RTT is greater than the remaining timeout for an operation, the
 	// operation is not sent to the server and no connections are closed.
 	mt.Run("minimum RTT used to prevent sending requests", func(mt *mtest.T) {
-		if testing.Short() {
-			t.Skip("skipping integration test in short mode")
-		}
+		mt.Parallel()
 
 		// Assert that we can call Ping with a 250ms timeout.
 		ctx, cancel := context.WithTimeout(context.Background(), 250*time.Millisecond)
@@ -557,16 +567,23 @@ func TestClient(t *testing.T) {
 
 		// Reset the client with a dialer that delays all network round trips by 300ms and set the
 		// heartbeat interval to 100ms to reduce the time it takes to collect RTT samples.
-		tpm := newTestPoolMonitor()
+		tpm := monitor.NewTestPoolMonitor()
 		mt.ResetClient(options.Client().
 			SetPoolMonitor(tpm.PoolMonitor).
-			SetDialer(newSlowConnDialer(300 * time.Millisecond)).
-			SetHeartbeatInterval(100 * time.Millisecond))
+			SetDialer(newSlowConnDialer(slowConnDialerDelay)).
+			SetHeartbeatInterval(reducedHeartbeatInterval))
 
 		// Assert that the minimum RTT is eventually >250ms.
 		topo := getTopologyFromClient(mt.Client)
-		assert.Soon(mt, func() {
+		helpers.AssertSoon(mt, func(ctx context.Context) {
 			for {
+				// Stop loop if callback has been canceled.
+				select {
+				case <-ctx.Done():
+					return
+				default:
+				}
+
 				time.Sleep(100 * time.Millisecond)
 
 				// Wait for all of the server's minimum RTTs to be >250ms.
@@ -574,7 +591,7 @@ func TestClient(t *testing.T) {
 				for _, desc := range topo.Description().Servers {
 					server, err := topo.FindServer(desc)
 					assert.Nil(mt, err, "FindServer error: %v", err)
-					if server.MinRTT() <= 250*time.Millisecond {
+					if server.RTTMonitor().Min() <= 250*time.Millisecond {
 						done = false
 					}
 				}
@@ -597,13 +614,306 @@ func TestClient(t *testing.T) {
 		closed := len(tpm.Events(func(e *event.PoolEvent) bool { return e.Type == event.ConnectionClosed }))
 		assert.Equal(t, 0, closed, "expected no connections to be closed")
 	})
+
+	mt.Run("RTT90 is monitored", func(mt *mtest.T) {
+		mt.Parallel()
+
+		// Reset the client with a dialer that delays all network round trips by 300ms and set the
+		// heartbeat interval to 100ms to reduce the time it takes to collect RTT samples.
+		mt.ResetClient(options.Client().
+			SetDialer(newSlowConnDialer(slowConnDialerDelay)).
+			SetHeartbeatInterval(reducedHeartbeatInterval))
+
+		// Assert that RTT90s are eventually >300ms.
+		topo := getTopologyFromClient(mt.Client)
+		helpers.AssertSoon(mt, func(ctx context.Context) {
+			for {
+				// Stop loop if callback has been canceled.
+				select {
+				case <-ctx.Done():
+					return
+				default:
+				}
+
+				time.Sleep(100 * time.Millisecond)
+
+				// Wait for all of the server's RTT90s to be >300ms.
+				done := true
+				for _, desc := range topo.Description().Servers {
+					server, err := topo.FindServer(desc)
+					assert.Nil(mt, err, "FindServer error: %v", err)
+					if server.RTTMonitor().P90() <= 300*time.Millisecond {
+						done = false
+					}
+				}
+				if done {
+					return
+				}
+			}
+		}, 10*time.Second)
+	})
+
+	// Test that if Timeout is set and the RTT90 is greater than the remaining timeout for an operation, the
+	// operation is not sent to the server, fails with a timeout error, and no connections are closed.
+	mt.Run("RTT90 used to prevent sending requests", func(mt *mtest.T) {
+		mt.Parallel()
+
+		// Assert that we can call Ping with a 250ms timeout.
+		ctx, cancel := context.WithTimeout(context.Background(), 250*time.Millisecond)
+		defer cancel()
+		err := mt.Client.Ping(ctx, nil)
+		assert.Nil(mt, err, "Ping error: %v", err)
+
+		// Reset the client with a dialer that delays all network round trips by 300ms, set the
+		// heartbeat interval to 100ms to reduce the time it takes to collect RTT samples, and
+		// set a Timeout of 0 (infinite) on the Client to ensure that RTT90 is used as a sending
+		// threshold.
+		tpm := monitor.NewTestPoolMonitor()
+		mt.ResetClient(options.Client().
+			SetPoolMonitor(tpm.PoolMonitor).
+			SetDialer(newSlowConnDialer(slowConnDialerDelay)).
+			SetHeartbeatInterval(reducedHeartbeatInterval).
+			SetTimeout(0))
+
+		// Assert that RTT90s are eventually >275ms.
+		topo := getTopologyFromClient(mt.Client)
+		helpers.AssertSoon(mt, func(ctx context.Context) {
+			for {
+				// Stop loop if callback has been canceled.
+				select {
+				case <-ctx.Done():
+					return
+				default:
+				}
+
+				time.Sleep(100 * time.Millisecond)
+
+				// Wait for all of the server's RTT90s to be >275ms.
+				done := true
+				for _, desc := range topo.Description().Servers {
+					server, err := topo.FindServer(desc)
+					assert.Nil(mt, err, "FindServer error: %v", err)
+					if server.RTTMonitor().P90() <= 275*time.Millisecond {
+						done = false
+					}
+				}
+				if done {
+					return
+				}
+			}
+		}, 10*time.Second)
+
+		// Once we've waited for the RTT90 for the servers to be >275ms, run 10 Ping operations
+		// with a timeout of 275ms and expect that they return timeout errors.
+		for i := 0; i < 10; i++ {
+			ctx, cancel = context.WithTimeout(context.Background(), 275*time.Millisecond)
+			err := mt.Client.Ping(ctx, nil)
+			cancel()
+			assert.NotNil(mt, err, "expected Ping to return an error")
+			assert.True(mt, mongo.IsTimeout(err), "expected a timeout error: got %v", err)
+		}
+
+		// Assert that the Ping timeouts result in no connections being closed.
+		closed := len(tpm.Events(func(e *event.PoolEvent) bool { return e.Type == event.ConnectionClosed }))
+		assert.Equal(t, 0, closed, "expected no connections to be closed")
+	})
+
+	// Test that OP_MSG is used for authentication-related commands on 3.6+ (WV 6+). Do not test when API version is
+	// set, as handshakes will always use OP_MSG.
+	opMsgOpts := mtest.NewOptions().ClientType(mtest.Proxy).MinServerVersion("3.6").Auth(true).RequireAPIVersion(false)
+	mt.RunOpts("OP_MSG used for authentication on 3.6+", opMsgOpts, func(mt *mtest.T) {
+		err := mt.Client.Ping(context.Background(), mtest.PrimaryRp)
+		assert.Nil(mt, err, "Ping error: %v", err)
+
+		msgPairs := mt.GetProxiedMessages()
+		assert.True(mt, len(msgPairs) >= 3, "expected at least 3 events, got %v", len(msgPairs))
+
+		// First message should a be connection handshake. This handshake should use OP_QUERY as the OpCode, as wire
+		// version is not yet known.
+		pair := msgPairs[0]
+		assert.Equal(mt, internal.LegacyHello, pair.CommandName, "expected command name %s at index 0, got %s",
+			internal.LegacyHello, pair.CommandName)
+		assert.Equal(mt, wiremessage.OpQuery, pair.Sent.OpCode,
+			"expected 'OP_QUERY' OpCode in wire message, got %q", pair.Sent.OpCode.String())
+
+		// Look for a saslContinue in the remaining proxied messages and assert that it uses the OP_MSG OpCode, as wire
+		// version is now known to be >= 6.
+		var saslContinueFound bool
+		for _, pair := range msgPairs[1:] {
+			if pair.CommandName == "saslContinue" {
+				saslContinueFound = true
+				assert.Equal(mt, wiremessage.OpMsg, pair.Sent.OpCode,
+					"expected 'OP_MSG' OpCode in wire message, got %s", pair.Sent.OpCode.String())
+				break
+			}
+		}
+		assert.True(mt, saslContinueFound, "did not find 'saslContinue' command in proxied messages")
+	})
+
+	// Test that OP_MSG is used for handshakes when API version is declared.
+	opMsgSAPIOpts := mtest.NewOptions().ClientType(mtest.Proxy).MinServerVersion("5.0").RequireAPIVersion(true)
+	mt.RunOpts("OP_MSG used for handshakes when API version declared", opMsgSAPIOpts, func(mt *mtest.T) {
+		err := mt.Client.Ping(context.Background(), mtest.PrimaryRp)
+		assert.Nil(mt, err, "Ping error: %v", err)
+
+		msgPairs := mt.GetProxiedMessages()
+		assert.True(mt, len(msgPairs) >= 3, "expected at least 3 events, got %v", len(msgPairs))
+
+		// First three messages should be connection handshakes: one for the heartbeat connection, another for the
+		// application connection, and a final one for the RTT monitor connection.
+		for idx, pair := range msgPairs[:3] {
+			assert.Equal(mt, "hello", pair.CommandName, "expected command name 'hello' at index %d, got %s", idx,
+				pair.CommandName)
+
+			// Assert that appended OpCode is OP_MSG when API version is set.
+			assert.Equal(mt, wiremessage.OpMsg, pair.Sent.OpCode,
+				"expected 'OP_MSG' OpCode in wire message, got %q", pair.Sent.OpCode.String())
+		}
+	})
+
+	// Test that OP_MSG is used for handshakes when loadBalanced is true.
+	opMsgLBOpts := mtest.NewOptions().ClientType(mtest.Proxy).MinServerVersion("5.0").Topologies(mtest.LoadBalanced)
+	mt.RunOpts("OP_MSG used for handshakes when loadBalanced is true", opMsgLBOpts, func(mt *mtest.T) {
+		err := mt.Client.Ping(context.Background(), mtest.PrimaryRp)
+		assert.Nil(mt, err, "Ping error: %v", err)
+
+		msgPairs := mt.GetProxiedMessages()
+		assert.True(mt, len(msgPairs) >= 3, "expected at least 3 events, got %v", len(msgPairs))
+
+		// First three messages should be connection handshakes: one for the heartbeat connection, another for the
+		// application connection, and a final one for the RTT monitor connection.
+		for idx, pair := range msgPairs[:3] {
+			assert.Equal(mt, "hello", pair.CommandName, "expected command name 'hello' at index %d, got %s", idx,
+				pair.CommandName)
+
+			// Assert that appended OpCode is OP_MSG when loadBalanced is true.
+			assert.Equal(mt, wiremessage.OpMsg, pair.Sent.OpCode,
+				"expected 'OP_MSG' OpCode in wire message, got %q", pair.Sent.OpCode.String())
+		}
+	})
+}
+
+func TestClient_BSONOptions(t *testing.T) {
+	t.Parallel()
+
+	mt := mtest.New(t, noClientOpts)
+	defer mt.Close()
+
+	type jsonTagsTest struct {
+		A string
+		B string `json:"x"`
+		C string `json:"y" bson:"3"`
+	}
+
+	testCases := []struct {
+		name       string
+		bsonOpts   *options.BSONOptions
+		doc        interface{}
+		decodeInto func() interface{}
+		want       interface{}
+		wantRaw    bson.Raw
+	}{
+		{
+			name: "UseJSONStructTags",
+			bsonOpts: &options.BSONOptions{
+				UseJSONStructTags: true,
+			},
+			doc: jsonTagsTest{
+				A: "apple",
+				B: "banana",
+				C: "carrot",
+			},
+			decodeInto: func() interface{} { return &jsonTagsTest{} },
+			want: &jsonTagsTest{
+				A: "apple",
+				B: "banana",
+				C: "carrot",
+			},
+			wantRaw: bson.Raw(bsoncore.NewDocumentBuilder().
+				AppendString("a", "apple").
+				AppendString("x", "banana").
+				AppendString("3", "carrot").
+				Build()),
+		},
+		{
+			name: "IntMinSize",
+			bsonOpts: &options.BSONOptions{
+				IntMinSize: true,
+			},
+			doc:        bson.D{{Key: "x", Value: int64(1)}},
+			decodeInto: func() interface{} { return &bson.D{} },
+			want:       &bson.D{{Key: "x", Value: int32(1)}},
+			wantRaw: bson.Raw(bsoncore.NewDocumentBuilder().
+				AppendInt32("x", 1).
+				Build()),
+		},
+		{
+			name: "DefaultDocumentM",
+			bsonOpts: &options.BSONOptions{
+				DefaultDocumentM: true,
+			},
+			doc:        bson.D{{Key: "doc", Value: bson.D{{Key: "a", Value: int64(1)}}}},
+			decodeInto: func() interface{} { return &bson.D{} },
+			want:       &bson.D{{Key: "doc", Value: bson.M{"a": int64(1)}}},
+		},
+	}
+
+	for _, tc := range testCases {
+		opts := mtest.NewOptions().ClientOptions(
+			options.Client().SetBSONOptions(tc.bsonOpts))
+		mt.RunOpts(tc.name, opts, func(mt *mtest.T) {
+			res, err := mt.Coll.InsertOne(context.Background(), tc.doc)
+			require.NoError(mt, err, "InsertOne error")
+
+			sr := mt.Coll.FindOne(
+				context.Background(),
+				bson.D{{Key: "_id", Value: res.InsertedID}},
+				// Exclude the auto-generated "_id" field so we can make simple
+				// assertions on the return value.
+				options.FindOne().SetProjection(bson.D{{Key: "_id", Value: 0}}))
+
+			if tc.want != nil {
+				got := tc.decodeInto()
+				err := sr.Decode(got)
+				require.NoError(mt, err, "Decode error")
+
+				assert.Equal(mt, tc.want, got, "expected and actual decoded result are different")
+			}
+
+			if tc.wantRaw != nil {
+				got, err := sr.DecodeBytes()
+				require.NoError(mt, err, "DecodeBytes error")
+
+				assert.EqualBSON(mt, tc.wantRaw, got)
+			}
+		})
+	}
+
+	opts := mtest.NewOptions().ClientOptions(
+		options.Client().SetBSONOptions(&options.BSONOptions{
+			ErrorOnInlineDuplicates: true,
+		}))
+	mt.RunOpts("ErrorOnInlineDuplicates", opts, func(mt *mtest.T) {
+		type inlineDupInner struct {
+			A string
+		}
+
+		type inlineDupOuter struct {
+			A string
+			B *inlineDupInner `bson:"b,inline"`
+		}
+
+		_, err := mt.Coll.InsertOne(context.Background(), inlineDupOuter{
+			A: "outer",
+			B: &inlineDupInner{
+				A: "inner",
+			},
+		})
+		require.Error(mt, err, "expected InsertOne to return an error")
+	})
 }
 
 func TestClientStress(t *testing.T) {
-	if testing.Short() {
-		t.Skip("skipping integration test in short mode")
-	}
-
 	mtOpts := mtest.NewOptions().CreateClient(false)
 	mt := mtest.New(t, mtOpts)
 	defer mt.Close()
@@ -658,7 +968,7 @@ func TestClientStress(t *testing.T) {
 		// pool configurations.
 		maxPoolSizes := []uint64{1, 10, 100}
 		for _, maxPoolSize := range maxPoolSizes {
-			tpm := newTestPoolMonitor()
+			tpm := monitor.NewTestPoolMonitor()
 			maxPoolSizeOpt := mtest.NewOptions().ClientOptions(
 				options.Client().
 					SetPoolMonitor(tpm.PoolMonitor).
