@@ -18,8 +18,9 @@ import (
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/bsontype"
 	"go.mongodb.org/mongo-driver/bson/primitive"
+	"go.mongodb.org/mongo-driver/internal/assert"
 	"go.mongodb.org/mongo-driver/internal/testutil"
-	"go.mongodb.org/mongo-driver/internal/testutil/assert"
+	"go.mongodb.org/mongo-driver/internal/testutil/helpers"
 	"go.mongodb.org/mongo-driver/mongo"
 	"go.mongodb.org/mongo-driver/mongo/gridfs"
 	"go.mongodb.org/mongo-driver/mongo/integration/mtest"
@@ -104,8 +105,8 @@ func killSessions(mt *mtest.T) {
 
 	// killAllSessions has to be run against each mongos in a sharded cluster, so we use the runCommandOnAllServers
 	// helper.
-	err := runCommandOnAllServers(mt, func(client *mongo.Client) error {
-		return client.Database("admin").RunCommand(mtest.Background, cmd, runCmdOpts).Err()
+	err := runCommandOnAllServers(func(client *mongo.Client) error {
+		return client.Database("admin").RunCommand(context.Background(), cmd, runCmdOpts).Err()
 	})
 
 	if err == nil {
@@ -118,28 +119,28 @@ func killSessions(mt *mtest.T) {
 
 // Utility function to run a command on all servers. For standalones, the command is run against the one server. For
 // replica sets, the command is run against the primary. sharded clusters, the command is run against each mongos.
-func runCommandOnAllServers(mt *mtest.T, commandFn func(client *mongo.Client) error) error {
+func runCommandOnAllServers(commandFn func(client *mongo.Client) error) error {
 	opts := options.Client().ApplyURI(mtest.ClusterURI())
 	testutil.AddTestServerAPIVersion(opts)
 
 	if mtest.ClusterTopologyKind() != mtest.Sharded {
-		client, err := mongo.Connect(mtest.Background, opts)
+		client, err := mongo.Connect(context.Background(), opts)
 		if err != nil {
 			return fmt.Errorf("error creating replica set client: %v", err)
 		}
-		defer func() { _ = client.Disconnect(mtest.Background) }()
+		defer func() { _ = client.Disconnect(context.Background()) }()
 
 		return commandFn(client)
 	}
 
 	for _, host := range opts.Hosts {
-		shardClient, err := mongo.Connect(mtest.Background, opts.SetHosts([]string{host}))
+		shardClient, err := mongo.Connect(context.Background(), opts.SetHosts([]string{host}))
 		if err != nil {
 			return fmt.Errorf("error creating client for mongos %v: %v", host, err)
 		}
 
 		err = commandFn(shardClient)
-		_ = shardClient.Disconnect(mtest.Background)
+		_ = shardClient.Disconnect(context.Background())
 		if err != nil {
 			return err
 		}
@@ -171,7 +172,7 @@ func executeAggregate(mt *mtest.T, agg aggregator, sess mongo.Session, args bson
 
 		switch key {
 		case "pipeline":
-			pipeline = rawArrayToInterfaceSlice(val.Array())
+			pipeline = helpers.RawToInterfaces(helpers.RawToDocuments(val.Array())...)
 		case "batchSize":
 			opts.SetBatchSize(val.Int32())
 		case "collation":
@@ -188,14 +189,14 @@ func executeAggregate(mt *mtest.T, agg aggregator, sess mongo.Session, args bson
 
 	if sess != nil {
 		var cur *mongo.Cursor
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var aerr error
 			cur, aerr = agg.Aggregate(sc, pipeline, opts)
 			return aerr
 		})
 		return cur, err
 	}
-	return agg.Aggregate(mtest.Background, pipeline, opts)
+	return agg.Aggregate(context.Background(), pipeline, opts)
 }
 
 func executeWatch(mt *mtest.T, w watcher, sess mongo.Session, args bson.Raw) (*mongo.ChangeStream, error) {
@@ -209,7 +210,7 @@ func executeWatch(mt *mtest.T, w watcher, sess mongo.Session, args bson.Raw) (*m
 
 		switch key {
 		case "pipeline":
-			pipeline = rawArrayToInterfaceSlice(val.Array())
+			pipeline = helpers.RawToInterfaces(helpers.RawToDocuments(val.Array())...)
 		default:
 			mt.Fatalf("unrecognized watch option: %v", key)
 		}
@@ -217,14 +218,14 @@ func executeWatch(mt *mtest.T, w watcher, sess mongo.Session, args bson.Raw) (*m
 
 	if sess != nil {
 		var stream *mongo.ChangeStream
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var csErr error
 			stream, csErr = w.Watch(sc, pipeline)
 			return csErr
 		})
 		return stream, err
 	}
-	return w.Watch(mtest.Background, pipeline)
+	return w.Watch(context.Background(), pipeline)
 }
 
 func executeCountDocuments(mt *mtest.T, sess mongo.Session, args bson.Raw) (int64, error) {
@@ -255,14 +256,14 @@ func executeCountDocuments(mt *mtest.T, sess mongo.Session, args bson.Raw) (int6
 
 	if sess != nil {
 		var count int64
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var countErr error
 			count, countErr = mt.Coll.CountDocuments(sc, filter, opts)
 			return countErr
 		})
 		return count, err
 	}
-	return mt.Coll.CountDocuments(mtest.Background, filter, opts)
+	return mt.Coll.CountDocuments(context.Background(), filter, opts)
 }
 
 func executeInsertOne(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.InsertOneResult, error) {
@@ -289,14 +290,14 @@ func executeInsertOne(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.In
 
 	if sess != nil {
 		var res *mongo.InsertOneResult
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var insertErr error
 			res, insertErr = mt.Coll.InsertOne(sc, doc, opts)
 			return insertErr
 		})
 		return res, err
 	}
-	return mt.Coll.InsertOne(mtest.Background, doc, opts)
+	return mt.Coll.InsertOne(context.Background(), doc, opts)
 }
 
 func executeInsertMany(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.InsertManyResult, error) {
@@ -312,7 +313,7 @@ func executeInsertMany(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.I
 
 		switch key {
 		case "documents":
-			docs = rawArrayToInterfaceSlice(val.Array())
+			docs = helpers.RawToInterfaces(helpers.RawToDocuments(val.Array())...)
 		case "options":
 			// Some of the older tests use this to set the "ordered" option
 			optsDoc := val.Document()
@@ -327,17 +328,17 @@ func executeInsertMany(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.I
 
 	if sess != nil {
 		var res *mongo.InsertManyResult
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var insertErr error
 			res, insertErr = mt.Coll.InsertMany(sc, docs, opts)
 			return insertErr
 		})
 		return res, err
 	}
-	return mt.Coll.InsertMany(mtest.Background, docs, opts)
+	return mt.Coll.InsertMany(context.Background(), docs, opts)
 }
 
-func setFindModifiers(mt *mtest.T, modifiersDoc bson.Raw, opts *options.FindOptions) {
+func setFindModifiers(modifiersDoc bson.Raw, opts *options.FindOptions) {
 	elems, _ := modifiersDoc.Elements()
 	for _, elem := range elems {
 		key := elem.Key()
@@ -387,7 +388,7 @@ func executeFind(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.Cursor,
 		case "collation":
 			opts = opts.SetCollation(createCollation(mt, val.Document()))
 		case "modifiers":
-			setFindModifiers(mt, val.Document(), opts)
+			setFindModifiers(val.Document(), opts)
 		case "allowDiskUse":
 			opts = opts.SetAllowDiskUse(val.Boolean())
 		case "projection":
@@ -400,14 +401,14 @@ func executeFind(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.Cursor,
 
 	if sess != nil {
 		var c *mongo.Cursor
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var findErr error
 			c, findErr = mt.Coll.Find(sc, filter, opts)
 			return findErr
 		})
 		return c, err
 	}
-	return mt.Coll.Find(mtest.Background, filter, opts)
+	return mt.Coll.Find(context.Background(), filter, opts)
 }
 
 func executeRunCommand(mt *mtest.T, sess mongo.Session, args bson.Raw) *mongo.SingleResult {
@@ -434,13 +435,13 @@ func executeRunCommand(mt *mtest.T, sess mongo.Session, args bson.Raw) *mongo.Si
 
 	if sess != nil {
 		var sr *mongo.SingleResult
-		_ = mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		_ = mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			sr = mt.DB.RunCommand(sc, cmd, opts)
 			return nil
 		})
 		return sr
 	}
-	return mt.DB.RunCommand(mtest.Background, cmd, opts)
+	return mt.DB.RunCommand(context.Background(), cmd, opts)
 }
 
 func executeListCollections(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.Cursor, error) {
@@ -462,14 +463,14 @@ func executeListCollections(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mo
 
 	if sess != nil {
 		var c *mongo.Cursor
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var lcErr error
 			c, lcErr = mt.DB.ListCollections(sc, filter)
 			return lcErr
 		})
 		return c, err
 	}
-	return mt.DB.ListCollections(mtest.Background, filter)
+	return mt.DB.ListCollections(context.Background(), filter)
 }
 
 func executeListCollectionNames(mt *mtest.T, sess mongo.Session, args bson.Raw) ([]string, error) {
@@ -491,14 +492,14 @@ func executeListCollectionNames(mt *mtest.T, sess mongo.Session, args bson.Raw)
 
 	if sess != nil {
 		var res []string
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var lcErr error
 			res, lcErr = mt.DB.ListCollectionNames(sc, filter)
 			return lcErr
 		})
 		return res, err
 	}
-	return mt.DB.ListCollectionNames(mtest.Background, filter)
+	return mt.DB.ListCollectionNames(context.Background(), filter)
 }
 
 func executeListDatabaseNames(mt *mtest.T, sess mongo.Session, args bson.Raw) ([]string, error) {
@@ -520,14 +521,14 @@ func executeListDatabaseNames(mt *mtest.T, sess mongo.Session, args bson.Raw) ([
 
 	if sess != nil {
 		var res []string
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var ldErr error
 			res, ldErr = mt.Client.ListDatabaseNames(sc, filter)
 			return ldErr
 		})
 		return res, err
 	}
-	return mt.Client.ListDatabaseNames(mtest.Background, filter)
+	return mt.Client.ListDatabaseNames(context.Background(), filter)
 }
 
 func executeListDatabases(mt *mtest.T, sess mongo.Session, args bson.Raw) (mongo.ListDatabasesResult, error) {
@@ -549,14 +550,14 @@ func executeListDatabases(mt *mtest.T, sess mongo.Session, args bson.Raw) (mongo
 
 	if sess != nil {
 		var res mongo.ListDatabasesResult
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var ldErr error
 			res, ldErr = mt.Client.ListDatabases(sc, filter)
 			return ldErr
 		})
 		return res, err
 	}
-	return mt.Client.ListDatabases(mtest.Background, filter)
+	return mt.Client.ListDatabases(context.Background(), filter)
 }
 
 func executeFindOne(mt *mtest.T, sess mongo.Session, args bson.Raw) *mongo.SingleResult {
@@ -578,13 +579,13 @@ func executeFindOne(mt *mtest.T, sess mongo.Session, args bson.Raw) *mongo.Singl
 
 	if sess != nil {
 		var res *mongo.SingleResult
-		_ = mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		_ = mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			res = mt.Coll.FindOne(sc, filter)
 			return nil
 		})
 		return res
 	}
-	return mt.Coll.FindOne(mtest.Background, filter)
+	return mt.Coll.FindOne(context.Background(), filter)
 }
 
 func executeListIndexes(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.Cursor, error) {
@@ -594,14 +595,14 @@ func executeListIndexes(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.
 	assert.Equal(mt, 0, len(args), "unexpected listIndexes arguments: %v", args)
 	if sess != nil {
 		var cursor *mongo.Cursor
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var listErr error
 			cursor, listErr = mt.Coll.Indexes().List(sc)
 			return listErr
 		})
 		return cursor, err
 	}
-	return mt.Coll.Indexes().List(mtest.Background)
+	return mt.Coll.Indexes().List(context.Background())
 }
 
 func executeDistinct(mt *mtest.T, sess mongo.Session, args bson.Raw) ([]interface{}, error) {
@@ -631,14 +632,14 @@ func executeDistinct(mt *mtest.T, sess mongo.Session, args bson.Raw) ([]interfac
 
 	if sess != nil {
 		var res []interface{}
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var derr error
 			res, derr = mt.Coll.Distinct(sc, fieldName, filter, opts)
 			return derr
 		})
 		return res, err
 	}
-	return mt.Coll.Distinct(mtest.Background, fieldName, filter, opts)
+	return mt.Coll.Distinct(context.Background(), fieldName, filter, opts)
 }
 
 func executeFindOneAndDelete(mt *mtest.T, sess mongo.Session, args bson.Raw) *mongo.SingleResult {
@@ -671,13 +672,13 @@ func executeFindOneAndDelete(mt *mtest.T, sess mongo.Session, args bson.Raw) *mo
 
 	if sess != nil {
 		var res *mongo.SingleResult
-		_ = mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		_ = mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			res = mt.Coll.FindOneAndDelete(sc, filter, opts)
 			return nil
 		})
 		return res
 	}
-	return mt.Coll.FindOneAndDelete(mtest.Background, filter, opts)
+	return mt.Coll.FindOneAndDelete(context.Background(), filter, opts)
 }
 
 func executeFindOneAndUpdate(mt *mtest.T, sess mongo.Session, args bson.Raw) *mongo.SingleResult {
@@ -699,7 +700,7 @@ func executeFindOneAndUpdate(mt *mtest.T, sess mongo.Session, args bson.Raw) *mo
 			update = createUpdate(mt, val)
 		case "arrayFilters":
 			opts = opts.SetArrayFilters(options.ArrayFilters{
-				Filters: rawArrayToInterfaceSlice(val.Array()),
+				Filters: helpers.RawToInterfaces(helpers.RawToDocuments(val.Array())...),
 			})
 		case "sort":
 			opts = opts.SetSort(val.Document())
@@ -728,13 +729,13 @@ func executeFindOneAndUpdate(mt *mtest.T, sess mongo.Session, args bson.Raw) *mo
 
 	if sess != nil {
 		var res *mongo.SingleResult
-		_ = mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		_ = mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			res = mt.Coll.FindOneAndUpdate(sc, filter, update, opts)
 			return nil
 		})
 		return res
 	}
-	return mt.Coll.FindOneAndUpdate(mtest.Background, filter, update, opts)
+	return mt.Coll.FindOneAndUpdate(context.Background(), filter, update, opts)
 }
 
 func executeFindOneAndReplace(mt *mtest.T, sess mongo.Session, args bson.Raw) *mongo.SingleResult {
@@ -781,13 +782,13 @@ func executeFindOneAndReplace(mt *mtest.T, sess mongo.Session, args bson.Raw) *m
 
 	if sess != nil {
 		var res *mongo.SingleResult
-		_ = mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		_ = mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			res = mt.Coll.FindOneAndReplace(sc, filter, replacement, opts)
 			return nil
 		})
 		return res
 	}
-	return mt.Coll.FindOneAndReplace(mtest.Background, filter, replacement, opts)
+	return mt.Coll.FindOneAndReplace(context.Background(), filter, replacement, opts)
 }
 
 func executeDeleteOne(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.DeleteResult, error) {
@@ -816,14 +817,14 @@ func executeDeleteOne(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.De
 
 	if sess != nil {
 		var res *mongo.DeleteResult
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var derr error
 			res, derr = mt.Coll.DeleteOne(sc, filter, opts)
 			return derr
 		})
 		return res, err
 	}
-	return mt.Coll.DeleteOne(mtest.Background, filter, opts)
+	return mt.Coll.DeleteOne(context.Background(), filter, opts)
 }
 
 func executeDeleteMany(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.DeleteResult, error) {
@@ -852,14 +853,14 @@ func executeDeleteMany(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.D
 
 	if sess != nil {
 		var res *mongo.DeleteResult
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var derr error
 			res, derr = mt.Coll.DeleteMany(sc, filter, opts)
 			return derr
 		})
 		return res, err
 	}
-	return mt.Coll.DeleteMany(mtest.Background, filter, opts)
+	return mt.Coll.DeleteMany(context.Background(), filter, opts)
 }
 
 func executeUpdateOne(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.UpdateResult, error) {
@@ -881,7 +882,7 @@ func executeUpdateOne(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.Up
 			update = createUpdate(mt, val)
 		case "arrayFilters":
 			opts = opts.SetArrayFilters(options.ArrayFilters{
-				Filters: rawArrayToInterfaceSlice(val.Array()),
+				Filters: helpers.RawToInterfaces(helpers.RawToDocuments(val.Array())...),
 			})
 		case "upsert":
 			opts = opts.SetUpsert(val.Boolean())
@@ -900,14 +901,14 @@ func executeUpdateOne(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.Up
 
 	if sess != nil {
 		var res *mongo.UpdateResult
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var uerr error
 			res, uerr = mt.Coll.UpdateOne(sc, filter, update, opts)
 			return uerr
 		})
 		return res, err
 	}
-	return mt.Coll.UpdateOne(mtest.Background, filter, update, opts)
+	return mt.Coll.UpdateOne(context.Background(), filter, update, opts)
 }
 
 func executeUpdateMany(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.UpdateResult, error) {
@@ -929,7 +930,7 @@ func executeUpdateMany(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.U
 			update = createUpdate(mt, val)
 		case "arrayFilters":
 			opts = opts.SetArrayFilters(options.ArrayFilters{
-				Filters: rawArrayToInterfaceSlice(val.Array()),
+				Filters: helpers.RawToInterfaces(helpers.RawToDocuments(val.Array())...),
 			})
 		case "upsert":
 			opts = opts.SetUpsert(val.Boolean())
@@ -948,14 +949,14 @@ func executeUpdateMany(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.U
 
 	if sess != nil {
 		var res *mongo.UpdateResult
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var uerr error
 			res, uerr = mt.Coll.UpdateMany(sc, filter, update, opts)
 			return uerr
 		})
 		return res, err
 	}
-	return mt.Coll.UpdateMany(mtest.Background, filter, update, opts)
+	return mt.Coll.UpdateMany(context.Background(), filter, update, opts)
 }
 
 func executeReplaceOne(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.UpdateResult, error) {
@@ -992,14 +993,14 @@ func executeReplaceOne(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.U
 
 	if sess != nil {
 		var res *mongo.UpdateResult
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var uerr error
 			res, uerr = mt.Coll.ReplaceOne(sc, filter, replacement, opts)
 			return uerr
 		})
 		return res, err
 	}
-	return mt.Coll.ReplaceOne(mtest.Background, filter, replacement, opts)
+	return mt.Coll.ReplaceOne(context.Background(), filter, replacement, opts)
 }
 
 type withTransactionArgs struct {
@@ -1076,14 +1077,14 @@ func executeBulkWrite(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.Bu
 
 	if sess != nil {
 		var res *mongo.BulkWriteResult
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var bwerr error
 			res, bwerr = mt.Coll.BulkWrite(sc, models, opts)
 			return bwerr
 		})
 		return res, err
 	}
-	return mt.Coll.BulkWrite(mtest.Background, models, opts)
+	return mt.Coll.BulkWrite(context.Background(), models, opts)
 }
 
 func createBulkWriteModels(mt *mtest.T, rawModels bson.Raw) []mongo.WriteModel {
@@ -1115,7 +1116,7 @@ func createBulkWriteModel(mt *mtest.T, rawModel bson.Raw) mongo.WriteModel {
 		}
 		if arrayFilters, err := args.LookupErr("arrayFilters"); err == nil {
 			uom.SetArrayFilters(options.ArrayFilters{
-				Filters: rawArrayToInterfaceSlice(arrayFilters.Array()),
+				Filters: helpers.RawToInterfaces(helpers.RawToDocuments(arrayFilters.Array())...),
 			})
 		}
 		if hintVal, err := args.LookupErr("hint"); err == nil {
@@ -1138,7 +1139,7 @@ func createBulkWriteModel(mt *mtest.T, rawModel bson.Raw) mongo.WriteModel {
 		}
 		if arrayFilters, err := args.LookupErr("arrayFilters"); err == nil {
 			umm.SetArrayFilters(options.ArrayFilters{
-				Filters: rawArrayToInterfaceSlice(arrayFilters.Array()),
+				Filters: helpers.RawToInterfaces(helpers.RawToDocuments(arrayFilters.Array())...),
 			})
 		}
 		if hintVal, err := args.LookupErr("hint"); err == nil {
@@ -1205,14 +1206,14 @@ func executeEstimatedDocumentCount(mt *mtest.T, sess mongo.Session, args bson.Ra
 
 	if sess != nil {
 		var res int64
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var countErr error
 			res, countErr = mt.Coll.EstimatedDocumentCount(sc)
 			return countErr
 		})
 		return res, err
 	}
-	return mt.Coll.EstimatedDocumentCount(mtest.Background)
+	return mt.Coll.EstimatedDocumentCount(context.Background())
 }
 
 func executeGridFSDownload(mt *mtest.T, bucket *gridfs.Bucket, args bson.Raw) (int64, error) {
@@ -1255,41 +1256,6 @@ func executeGridFSDownloadByName(mt *mtest.T, bucket *gridfs.Bucket, args bson.R
 	return bucket.DownloadToStreamByName(file, new(bytes.Buffer))
 }
 
-// returns the result from the operation and the name of the target collection
-func executeRenameCollection(mt *mtest.T, sess mongo.Session, args bson.Raw) (*mongo.SingleResult, string) {
-	mt.Helper()
-
-	var toName string
-	elems, _ := args.Elements()
-	for _, elem := range elems {
-		key := elem.Key()
-		opt := elem.Value()
-
-		switch key {
-		case "to":
-			toName = opt.StringValue()
-		default:
-			mt.Fatalf("unrecognized renameCollection option %v", key)
-		}
-	}
-
-	renameCmd := bson.D{
-		{"renameCollection", mt.DB.Name() + "." + mt.Coll.Name()},
-		{"to", mt.DB.Name() + "." + toName},
-	}
-	admin := mt.Client.Database("admin")
-
-	if sess != nil {
-		var res *mongo.SingleResult
-		_ = mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
-			res = admin.RunCommand(sc, renameCmd)
-			return nil
-		})
-		return res, toName
-	}
-	return admin.RunCommand(mtest.Background, renameCmd), toName
-}
-
 func executeCreateIndex(mt *mtest.T, sess mongo.Session, args bson.Raw) (string, error) {
 	mt.Helper()
 
@@ -1314,14 +1280,14 @@ func executeCreateIndex(mt *mtest.T, sess mongo.Session, args bson.Raw) (string,
 
 	if sess != nil {
 		var indexName string
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var indexErr error
 			indexName, indexErr = mt.Coll.Indexes().CreateOne(sc, model)
 			return indexErr
 		})
 		return indexName, err
 	}
-	return mt.Coll.Indexes().CreateOne(mtest.Background, model)
+	return mt.Coll.Indexes().CreateOne(context.Background(), model)
 }
 
 func executeDropIndex(mt *mtest.T, sess mongo.Session, args bson.Raw) (bson.Raw, error) {
@@ -1343,14 +1309,14 @@ func executeDropIndex(mt *mtest.T, sess mongo.Session, args bson.Raw) (bson.Raw,
 
 	if sess != nil {
 		var res bson.Raw
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			var indexErr error
 			res, indexErr = mt.Coll.Indexes().DropOne(sc, name)
 			return indexErr
 		})
 		return res, err
 	}
-	return mt.Coll.Indexes().DropOne(mtest.Background, name)
+	return mt.Coll.Indexes().DropOne(context.Background(), name)
 }
 
 func executeDropCollection(mt *mtest.T, sess mongo.Session, args bson.Raw) error {
@@ -1363,6 +1329,8 @@ func executeDropCollection(mt *mtest.T, sess mongo.Session, args bson.Raw) error
 		val := elem.Value()
 
 		switch key {
+		case "encryptedFields":
+			mt.Fatalf("unsupported field: encryptedFields")
 		case "collection":
 			collName = val.StringValue()
 		default:
@@ -1372,17 +1340,19 @@ func executeDropCollection(mt *mtest.T, sess mongo.Session, args bson.Raw) error
 
 	coll := mt.DB.Collection(collName)
 	if sess != nil {
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
 			return coll.Drop(sc)
 		})
 		return err
 	}
-	return coll.Drop(mtest.Background)
+	return coll.Drop(context.Background())
 }
 
 func executeCreateCollection(mt *mtest.T, sess mongo.Session, args bson.Raw) error {
 	mt.Helper()
 
+	cco := options.CreateCollection()
+
 	var collName string
 	elems, _ := args.Elements()
 	for _, elem := range elems {
@@ -1390,34 +1360,35 @@ func executeCreateCollection(mt *mtest.T, sess mongo.Session, args bson.Raw) err
 		val := elem.Value()
 
 		switch key {
+		case "encryptedFields":
+			cco.SetEncryptedFields(val.Document())
 		case "collection":
 			collName = val.StringValue()
+		case "validator":
+			cco.SetValidator(val.Document())
 		case "session":
 		default:
 			mt.Fatalf("unrecognized createCollection option %v", key)
 		}
 	}
 
-	createCmd := bson.D{
-		{"create", collName},
-	}
 	if sess != nil {
-		err := mongo.WithSession(mtest.Background, sess, func(sc mongo.SessionContext) error {
-			return mt.DB.RunCommand(sc, createCmd).Err()
+		err := mongo.WithSession(context.Background(), sess, func(sc mongo.SessionContext) error {
+			return mt.DB.CreateCollection(sc, collName, cco)
 		})
 		return err
 	}
-	return mt.DB.RunCommand(mtest.Background, createCmd).Err()
+	return mt.DB.CreateCollection(context.Background(), collName, cco)
 }
 
 func executeAdminCommand(mt *mtest.T, op *operation) {
 	// Per the streamable hello test format description, a separate client must be used to execute this operation.
 	clientOpts := options.Client().ApplyURI(mtest.ClusterURI())
 	testutil.AddTestServerAPIVersion(clientOpts)
-	client, err := mongo.Connect(mtest.Background, clientOpts)
+	client, err := mongo.Connect(context.Background(), clientOpts)
 	assert.Nil(mt, err, "Connect error: %v", err)
 	defer func() {
-		_ = client.Disconnect(mtest.Background)
+		_ = client.Disconnect(context.Background())
 	}()
 
 	cmd := op.Arguments.Lookup("command").Document()
@@ -1441,14 +1412,14 @@ func executeAdminCommand(mt *mtest.T, op *operation) {
 	}
 
 	db := client.Database("admin")
-	err = db.RunCommand(mtest.Background, cmd, rco).Err()
+	err = db.RunCommand(context.Background(), cmd, rco).Err()
 	assert.Nil(mt, err, "RunCommand error for command %q: %v", op.CommandName, err)
 }
 
 func executeAdminCommandWithRetry(mt *mtest.T, client *mongo.Client, cmd interface{}, opts ...*options.RunCmdOptions) {
 	mt.Helper()
 
-	ctx, cancel := context.WithTimeout(mtest.Background, 10*time.Second)
+	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
 	defer cancel()
 
 	for {
@@ -1651,7 +1622,7 @@ func verifyCursorResult(mt *mtest.T, cur *mongo.Cursor, result interface{}) {
 	// To account for this, we fetch all documents via cursor.All and then compare them to the result if it's non-nil.
 	assert.NotNil(mt, cur, "expected cursor to not be nil")
 	var actual []bson.Raw
-	err := cur.All(mtest.Background, &actual)
+	err := cur.All(context.Background(), &actual)
 	assert.Nil(mt, err, "All error: %v", err)
 
 	if result == nil {
