@@ -7,9 +7,11 @@
 package integration
 
 import (
+	"context"
 	"errors"
 	"fmt"
 	"io/ioutil"
+	"os"
 	"path"
 	"reflect"
 	"sync"
@@ -22,8 +24,9 @@ import (
 	"go.mongodb.org/mongo-driver/bson/bsonrw"
 	"go.mongodb.org/mongo-driver/bson/bsontype"
 	"go.mongodb.org/mongo-driver/event"
+	"go.mongodb.org/mongo-driver/internal/assert"
 	"go.mongodb.org/mongo-driver/internal/testutil"
-	"go.mongodb.org/mongo-driver/internal/testutil/assert"
+	"go.mongodb.org/mongo-driver/internal/testutil/helpers"
 	"go.mongodb.org/mongo-driver/mongo"
 	"go.mongodb.org/mongo-driver/mongo/address"
 	"go.mongodb.org/mongo-driver/mongo/gridfs"
@@ -40,6 +43,7 @@ const (
 	gridFSChunks           = "fs.chunks"
 	spec1403SkipReason     = "servers less than 4.2 do not have mongocryptd; see SPEC-1403"
 	godriver2123SkipReason = "failpoints and timeouts together cause failures; see GODRIVER-2123"
+	godriver2413SkipReason = "encryptedFields argument is not supported on Collection.Drop; see GODRIVER-2413"
 )
 
 var (
@@ -54,18 +58,22 @@ var (
 		// i/o timeout.
 		"Ignore network timeout error on find":             godriver2123SkipReason,
 		"Network error on minPoolSize background creation": godriver2123SkipReason,
+		"CreateCollection from encryptedFields.":           godriver2413SkipReason,
+		"DropCollection from encryptedFields":              godriver2413SkipReason,
+		"DropCollection from remote encryptedFields":       godriver2413SkipReason,
 	}
 )
 
 type testFile struct {
-	RunOn          []mtest.RunOnBlock `bson:"runOn"`
-	DatabaseName   string             `bson:"database_name"`
-	CollectionName string             `bson:"collection_name"`
-	BucketName     string             `bson:"bucket_name"`
-	Data           testData           `bson:"data"`
-	JSONSchema     bson.Raw           `bson:"json_schema"`
-	KeyVaultData   []bson.Raw         `bson:"key_vault_data"`
-	Tests          []*testCase        `bson:"tests"`
+	RunOn           []mtest.RunOnBlock `bson:"runOn"`
+	DatabaseName    string             `bson:"database_name"`
+	CollectionName  string             `bson:"collection_name"`
+	BucketName      string             `bson:"bucket_name"`
+	Data            testData           `bson:"data"`
+	JSONSchema      bson.Raw           `bson:"json_schema"`
+	KeyVaultData    []bson.Raw         `bson:"key_vault_data"`
+	Tests           []*testCase        `bson:"tests"`
+	EncryptedFields bson.Raw           `bson:"encrypted_fields"`
 }
 
 type testData struct {
@@ -137,16 +145,19 @@ type operation struct {
 
 type expectation struct {
 	CommandStartedEvent *struct {
-		CommandName  string   `bson:"command_name"`
-		DatabaseName string   `bson:"database_name"`
-		Command      bson.Raw `bson:"command"`
+		CommandName  string                 `bson:"command_name"`
+		DatabaseName string                 `bson:"database_name"`
+		Command      bson.Raw               `bson:"command"`
+		Extra        map[string]interface{} `bson:",inline"`
 	} `bson:"command_started_event"`
 	CommandSucceededEvent *struct {
-		CommandName string   `bson:"command_name"`
-		Reply       bson.Raw `bson:"reply"`
+		CommandName string                 `bson:"command_name"`
+		Reply       bson.Raw               `bson:"reply"`
+		Extra       map[string]interface{} `bson:",inline"`
 	} `bson:"command_succeeded_event"`
 	CommandFailedEvent *struct {
-		CommandName string `bson:"command_name"`
+		CommandName string                 `bson:"command_name"`
+		Extra       map[string]interface{} `bson:",inline"`
 	} `bson:"command_failed_event"`
 }
 
@@ -166,13 +177,12 @@ type operationError struct {
 	ErrorLabelsOmit    []string `bson:"errorLabelsOmit"`
 }
 
-const dataPath string = "../../data/"
+const dataPath string = "../../testdata/"
 
 var directories = []string{
 	"transactions/legacy",
 	"convenient-transactions",
 	"retryable-reads",
-	"sessions/legacy",
 	"read-write-concern/operation",
 	"server-discovery-and-monitoring/integration",
 	"atlas-data-lake-testing",
@@ -227,15 +237,21 @@ func runSpecTestCase(mt *mtest.T, test *testCase, testFile testFile) {
 		// pin to a single mongos
 		opts = opts.ClientType(mtest.Pinned)
 	}
+
+	cco := options.CreateCollection()
 	if len(testFile.JSONSchema) > 0 {
 		validator := bson.D{
 			{"$jsonSchema", testFile.JSONSchema},
 		}
-		opts.CollectionCreateOptions(bson.D{
-			{"validator", validator},
-		})
+		cco.SetValidator(validator)
 	}
 
+	if len(testFile.EncryptedFields) > 0 {
+		cco.SetEncryptedFields(testFile.EncryptedFields)
+	}
+
+	opts.CollectionCreateOptions(cco)
+
 	// Start the test without setting client options so the setup will be done with a default client.
 	mt.RunOpts(test.Description, opts, func(mt *mtest.T) {
 		if len(test.SkipReason) > 0 {
@@ -247,9 +263,9 @@ func runSpecTestCase(mt *mtest.T, test *testCase, testFile testFile) {
 
 		// work around for SERVER-39704: run a non-transactional distinct against each shard in a sharded cluster
 		if mtest.ClusterTopologyKind() == mtest.Sharded && test.Description == "distinct" {
-			err := runCommandOnAllServers(mt, func(mongosClient *mongo.Client) error {
+			err := runCommandOnAllServers(func(mongosClient *mongo.Client) error {
 				coll := mongosClient.Database(mt.DB.Name()).Collection(mt.Coll.Name())
-				_, err := coll.Distinct(mtest.Background, "x", bson.D{})
+				_, err := coll.Distinct(context.Background(), "x", bson.D{})
 				return err
 			})
 			assert.Nil(mt, err, "error running distinct against all mongoses: %v", err)
@@ -268,6 +284,26 @@ func runSpecTestCase(mt *mtest.T, test *testCase, testFile testFile) {
 
 		// Reset the client using the client options specified in the test.
 		testClientOpts := createClientOptions(mt, test.ClientOptions)
+
+		// If AutoEncryptionOptions is set and AutoEncryption isn't disabled (neither
+		// bypassAutoEncryption nor bypassQueryAnalysis are true), then add extra options to load
+		// the crypt_shared library.
+		if testClientOpts.AutoEncryptionOptions != nil {
+			bypassAutoEncryption := testClientOpts.AutoEncryptionOptions.BypassAutoEncryption != nil &&
+				*testClientOpts.AutoEncryptionOptions.BypassAutoEncryption
+			bypassQueryAnalysis := testClientOpts.AutoEncryptionOptions.BypassQueryAnalysis != nil &&
+				*testClientOpts.AutoEncryptionOptions.BypassQueryAnalysis
+			if !bypassAutoEncryption && !bypassQueryAnalysis {
+				if testClientOpts.AutoEncryptionOptions.ExtraOptions == nil {
+					testClientOpts.AutoEncryptionOptions.ExtraOptions = make(map[string]interface{})
+				}
+
+				for k, v := range getCryptSharedLibExtraOptions() {
+					testClientOpts.AutoEncryptionOptions.ExtraOptions[k] = v
+				}
+			}
+		}
+
 		test.monitor = newUnifiedRunnerEventMonitor()
 		testClientOpts.SetPoolMonitor(&event.PoolMonitor{
 			Event: test.monitor.handlePoolEvent,
@@ -289,8 +325,8 @@ func runSpecTestCase(mt *mtest.T, test *testCase, testFile testFile) {
 		sess0, sess1 := setupSessions(mt, test)
 		if sess0 != nil {
 			defer func() {
-				sess0.EndSession(mtest.Background)
-				sess1.EndSession(mtest.Background)
+				sess0.EndSession(context.Background())
+				sess1.EndSession(context.Background())
 			}()
 		}
 
@@ -303,8 +339,8 @@ func runSpecTestCase(mt *mtest.T, test *testCase, testFile testFile) {
 
 		// Needs to be done here (in spite of defer) because some tests
 		// require end session to be called before we check expectation
-		sess0.EndSession(mtest.Background)
-		sess1.EndSession(mtest.Background)
+		sess0.EndSession(context.Background())
+		sess1.EndSession(context.Background())
 		mt.ClearFailPoints()
 
 		checkExpectations(mt, test.Expectations, sess0.ID(), sess1.ID())
@@ -433,13 +469,13 @@ func executeTestRunnerOperation(mt *mtest.T, testCase *testCase, op *operation,
 		targetHost := clientSession.PinnedServer.Addr.String()
 		opts := options.Client().ApplyURI(mtest.ClusterURI()).SetHosts([]string{targetHost})
 		testutil.AddTestServerAPIVersion(opts)
-		client, err := mongo.Connect(mtest.Background, opts)
+		client, err := mongo.Connect(context.Background(), opts)
 		if err != nil {
 			return fmt.Errorf("Connect error for targeted client: %v", err)
 		}
-		defer func() { _ = client.Disconnect(mtest.Background) }()
+		defer func() { _ = client.Disconnect(context.Background()) }()
 
-		if err = client.Database("admin").RunCommand(mtest.Background, fp).Err(); err != nil {
+		if err = client.Database("admin").RunCommand(context.Background(), fp).Err(); err != nil {
 			return fmt.Errorf("error setting targeted fail point: %v", err)
 		}
 		mt.TrackFailPoint(fp.ConfigureFailPoint)
@@ -472,10 +508,6 @@ func executeTestRunnerOperation(mt *mtest.T, testCase *testCase, op *operation,
 		if clientSession.PinnedServer != nil {
 			return fmt.Errorf("expected pinned server to be nil but got %q", clientSession.PinnedServer.Addr)
 		}
-	case "assertSessionDirty":
-		return verifyDirtySessionState(clientSession, true)
-	case "assertSessionNotDirty":
-		return verifyDirtySessionState(clientSession, false)
 	case "assertSameLsidOnLastTwoCommands":
 		first, second := lastTwoIDs(mt)
 		if !first.Equal(second) {
@@ -487,13 +519,13 @@ func executeTestRunnerOperation(mt *mtest.T, testCase *testCase, op *operation,
 			return fmt.Errorf("expected last two lsids to be not equal but both were %v", first)
 		}
 	case "assertCollectionExists":
-		return verifyCollectionState(mt, op, true)
+		return verifyCollectionState(op, true)
 	case "assertCollectionNotExists":
-		return verifyCollectionState(mt, op, false)
+		return verifyCollectionState(op, false)
 	case "assertIndexExists":
-		return verifyIndexState(mt, op, true)
+		return verifyIndexState(op, true)
 	case "assertIndexNotExists":
-		return verifyIndexState(mt, op, false)
+		return verifyIndexState(op, false)
 	case "wait":
 		time.Sleep(convertValueToMilliseconds(mt, op.Arguments.Lookup("ms")))
 	case "waitForEvent":
@@ -519,22 +551,12 @@ func executeTestRunnerOperation(mt *mtest.T, testCase *testCase, op *operation,
 	return nil
 }
 
-func verifyDirtySessionState(clientSession *session.Client, expectedDirty bool) error {
-	if clientSession.Server == nil {
-		return errors.New("expected valid server session, got nil")
-	}
-	if markedDirty := clientSession.Server.Dirty; markedDirty != expectedDirty {
-		return fmt.Errorf("expected server session to be marked dirty: %v, got %v", expectedDirty, markedDirty)
-	}
-	return nil
-}
-
-func verifyIndexState(mt *mtest.T, op *operation, shouldExist bool) error {
+func verifyIndexState(op *operation, shouldExist bool) error {
 	db := op.Arguments.Lookup("database").StringValue()
 	coll := op.Arguments.Lookup("collection").StringValue()
 	index := op.Arguments.Lookup("index").StringValue()
 
-	exists, err := indexExists(mt, db, coll, index)
+	exists, err := indexExists(db, coll, index)
 	if err != nil {
 		return err
 	}
@@ -545,16 +567,16 @@ func verifyIndexState(mt *mtest.T, op *operation, shouldExist bool) error {
 	return nil
 }
 
-func indexExists(mt *mtest.T, dbName, collName, indexName string) (bool, error) {
+func indexExists(dbName, collName, indexName string) (bool, error) {
 	// Use global client because listIndexes cannot be executed inside a transaction.
 	iv := mtest.GlobalClient().Database(dbName).Collection(collName).Indexes()
-	cursor, err := iv.List(mtest.Background)
+	cursor, err := iv.List(context.Background())
 	if err != nil {
 		return false, fmt.Errorf("IndexView.List error: %v", err)
 	}
-	defer cursor.Close(mtest.Background)
+	defer cursor.Close(context.Background())
 
-	for cursor.Next(mtest.Background) {
+	for cursor.Next(context.Background()) {
 		if cursor.Current.Lookup("name").StringValue() == indexName {
 			return true, nil
 		}
@@ -562,11 +584,11 @@ func indexExists(mt *mtest.T, dbName, collName, indexName string) (bool, error)
 	return false, cursor.Err()
 }
 
-func verifyCollectionState(mt *mtest.T, op *operation, shouldExist bool) error {
+func verifyCollectionState(op *operation, shouldExist bool) error {
 	db := op.Arguments.Lookup("database").StringValue()
 	coll := op.Arguments.Lookup("collection").StringValue()
 
-	exists, err := collectionExists(mt, db, coll)
+	exists, err := collectionExists(db, coll)
 	if err != nil {
 		return err
 	}
@@ -577,13 +599,13 @@ func verifyCollectionState(mt *mtest.T, op *operation, shouldExist bool) error {
 	return nil
 }
 
-func collectionExists(mt *mtest.T, dbName, collName string) (bool, error) {
+func collectionExists(dbName, collName string) (bool, error) {
 	filter := bson.D{
 		{"name", collName},
 	}
 
 	// Use global client because listCollections cannot be executed inside a transaction.
-	collections, err := mtest.GlobalClient().Database(dbName).ListCollectionNames(mtest.Background, filter)
+	collections, err := mtest.GlobalClient().Database(dbName).ListCollectionNames(context.Background(), filter)
 	if err != nil {
 		return false, fmt.Errorf("ListCollectionNames error: %v", err)
 	}
@@ -609,14 +631,11 @@ func executeSessionOperation(mt *mtest.T, op *operation, sess mongo.Session) err
 		}
 		return sess.StartTransaction(txnOpts)
 	case "commitTransaction":
-		return sess.CommitTransaction(mtest.Background)
+		return sess.CommitTransaction(context.Background())
 	case "abortTransaction":
-		return sess.AbortTransaction(mtest.Background)
+		return sess.AbortTransaction(context.Background())
 	case "withTransaction":
 		return executeWithTransaction(mt, sess, op.Arguments)
-	case "endSession":
-		sess.EndSession(mtest.Background)
-		return nil
 	default:
 		mt.Fatalf("unrecognized session operation: %v", op.Name)
 	}
@@ -654,7 +673,7 @@ func executeCollectionOperation(mt *mtest.T, op *operation, sess mongo.Session)
 		cursor, err := executeFind(mt, sess, op.Arguments)
 		if op.opError == nil && err == nil {
 			verifyCursorResult(mt, cursor, op.Result)
-			_ = cursor.Close(mtest.Background)
+			_ = cursor.Close(context.Background())
 		}
 		return err
 	case "findOneAndDelete":
@@ -709,7 +728,7 @@ func executeCollectionOperation(mt *mtest.T, op *operation, sess mongo.Session)
 		cursor, err := executeAggregate(mt, mt.Coll, sess, op.Arguments)
 		if op.opError == nil && err == nil {
 			verifyCursorResult(mt, cursor, op.Result)
-			_ = cursor.Close(mtest.Background)
+			_ = cursor.Close(context.Background())
 		}
 		return err
 	case "bulkWrite":
@@ -734,14 +753,14 @@ func executeCollectionOperation(mt *mtest.T, op *operation, sess mongo.Session)
 		cursor, err := executeListIndexes(mt, sess, op.Arguments)
 		if op.opError == nil && err == nil {
 			verifyCursorResult(mt, cursor, op.Result)
-			_ = cursor.Close(mtest.Background)
+			_ = cursor.Close(context.Background())
 		}
 		return err
 	case "watch":
 		stream, err := executeWatch(mt, mt.Coll, sess, op.Arguments)
 		if op.opError == nil && err == nil {
 			assert.Nil(mt, op.Result, "unexpected result for watch: %v", op.Result)
-			_ = stream.Close(mtest.Background)
+			_ = stream.Close(context.Background())
 		}
 		return err
 	case "createIndex":
@@ -779,14 +798,14 @@ func executeDatabaseOperation(mt *mtest.T, op *operation, sess mongo.Session) er
 		cursor, err := executeAggregate(mt, mt.DB, sess, op.Arguments)
 		if op.opError == nil && err == nil {
 			verifyCursorResult(mt, cursor, op.Result)
-			_ = cursor.Close(mtest.Background)
+			_ = cursor.Close(context.Background())
 		}
 		return err
 	case "listCollections":
 		cursor, err := executeListCollections(mt, sess, op.Arguments)
 		if op.opError == nil && err == nil {
 			assert.Nil(mt, op.Result, "unexpected result for listCollections: %v", op.Result)
-			_ = cursor.Close(mtest.Background)
+			_ = cursor.Close(context.Background())
 		}
 		return err
 	case "listCollectionNames":
@@ -799,7 +818,7 @@ func executeDatabaseOperation(mt *mtest.T, op *operation, sess mongo.Session) er
 		stream, err := executeWatch(mt, mt.DB, sess, op.Arguments)
 		if op.opError == nil && err == nil {
 			assert.Nil(mt, op.Result, "unexpected result for watch: %v", op.Result)
-			_ = stream.Close(mtest.Background)
+			_ = stream.Close(context.Background())
 		}
 		return err
 	case "dropCollection":
@@ -840,7 +859,7 @@ func executeClientOperation(mt *mtest.T, op *operation, sess mongo.Session) erro
 		stream, err := executeWatch(mt, mt.Client, sess, op.Arguments)
 		if op.opError == nil && err == nil {
 			assert.Nil(mt, op.Result, "unexpected result for watch: %v", op.Result)
-			_ = stream.Close(mtest.Background)
+			_ = stream.Close(context.Background())
 		}
 		return err
 	case "listDatabaseObjects":
@@ -873,12 +892,12 @@ func setupSessions(mt *mtest.T, test *testCase) (mongo.Session, mongo.Session) {
 func insertDocuments(mt *mtest.T, coll *mongo.Collection, rawDocs []bson.Raw) {
 	mt.Helper()
 
-	docsToInsert := rawSliceToInterfaceSlice(rawDocs)
+	docsToInsert := helpers.RawToInterfaces(rawDocs...)
 	if len(docsToInsert) == 0 {
 		return
 	}
 
-	_, err := coll.InsertMany(mtest.Background, docsToInsert)
+	_, err := coll.InsertMany(context.Background(), docsToInsert)
 	assert.Nil(mt, err, "InsertMany error for collection %v: %v", coll.Name(), err)
 }
 
@@ -888,6 +907,10 @@ func setupTest(mt *mtest.T, testFile *testFile, testCase *testCase) {
 
 	// key vault data
 	if len(testFile.KeyVaultData) > 0 {
+		// Drop the key vault collection in case it exists from a prior test run.
+		err := mt.Client.Database("keyvault").Collection("datakeys").Drop(context.Background())
+		assert.Nil(mt, err, "error dropping key vault collection")
+
 		keyVaultColl := mt.CreateCollection(mtest.Collection{
 			Name: "datakeys",
 			DB:   "keyvault",
@@ -937,7 +960,7 @@ func verifyTestOutcome(mt *mtest.T, outcomeColl *outcomeCollection) {
 
 	findOpts := options.Find().
 		SetSort(bson.M{"_id": 1})
-	cursor, err := coll.Find(mtest.Background, bson.D{}, findOpts)
+	cursor, err := coll.Find(context.Background(), bson.D{}, findOpts)
 	assert.Nil(mt, err, "Find error: %v", err)
 	verifyCursorResult(mt, cursor, outcomeColl.Data)
 }
@@ -948,3 +971,16 @@ func getTopologyFromClient(client *mongo.Client) *topology.Topology {
 	deploymentField = reflect.NewAt(deploymentField.Type(), unsafe.Pointer(deploymentField.UnsafeAddr())).Elem()
 	return deploymentField.Interface().(*topology.Topology)
 }
+
+// getCryptSharedLibExtraOptions returns an AutoEncryption extra options map with crypt_shared
+// library path information if the CRYPT_SHARED_LIB_PATH environment variable is set.
+func getCryptSharedLibExtraOptions() map[string]interface{} {
+	path := os.Getenv("CRYPT_SHARED_LIB_PATH")
+	if path == "" {
+		return nil
+	}
+	return map[string]interface{}{
+		"cryptSharedLibRequired": true,
+		"cryptSharedLibPath":     path,
+	}
+}
