@@ -4,6 +4,7 @@
 // not use this file except in compliance with the License. You may obtain
 // a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 
+//go:build !cse
 // +build !cse
 
 package mongocrypt
@@ -33,6 +34,6 @@ func (kc *KmsContext) BytesNeeded() int32 {
 }
 
 // FeedResponse feeds the bytes received from the KMS to mongocrypt.
-func (kc *KmsContext) FeedResponse(response []byte) error {
+func (kc *KmsContext) FeedResponse([]byte) error {
 	panic(cseNotSupportedMsg)
 }
