@@ -1,9 +1,16 @@
+// Copyright (C) MongoDB, Inc. 2022-present.
+//
+// Licensed under the Apache License, Version 2.0 (the "License"); you may
+// not use this file except in compliance with the License. You may obtain
+// a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
+
 package driver
 
 import (
 	"bytes"
 	"context"
 	"errors"
+	"math"
 	"testing"
 	"time"
 
@@ -11,7 +18,8 @@ import (
 	"go.mongodb.org/mongo-driver/bson/bsontype"
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/internal"
-	"go.mongodb.org/mongo-driver/internal/testutil/assert"
+	"go.mongodb.org/mongo-driver/internal/assert"
+	"go.mongodb.org/mongo-driver/internal/uuid"
 	"go.mongodb.org/mongo-driver/mongo/address"
 	"go.mongodb.org/mongo-driver/mongo/description"
 	"go.mongodb.org/mongo-driver/mongo/readconcern"
@@ -20,7 +28,6 @@ import (
 	"go.mongodb.org/mongo-driver/tag"
 	"go.mongodb.org/mongo-driver/x/bsonx/bsoncore"
 	"go.mongodb.org/mongo-driver/x/mongo/driver/session"
-	"go.mongodb.org/mongo-driver/x/mongo/driver/uuid"
 	"go.mongodb.org/mongo-driver/x/mongo/driver/wiremessage"
 )
 
@@ -97,7 +104,7 @@ func TestOperation(t *testing.T) {
 		}{
 			{"CommandFn", &Operation{}, InvalidOperationError{MissingField: "CommandFn"}},
 			{"Deployment", &Operation{CommandFn: cmdFn}, InvalidOperationError{MissingField: "Deployment"}},
-			{"Database", &Operation{CommandFn: cmdFn, Deployment: d}, InvalidOperationError{MissingField: "Database"}},
+			{"Database", &Operation{CommandFn: cmdFn, Deployment: d}, errDatabaseNameEmpty},
 			{"<nil>", &Operation{CommandFn: cmdFn, Deployment: d, Database: "test"}, nil},
 		}
 
@@ -119,15 +126,15 @@ func TestOperation(t *testing.T) {
 		id, err := uuid.New()
 		noerr(t, err)
 
-		sess, err := session.NewClientSession(sessPool, id, session.Explicit)
+		sess, err := session.NewClientSession(sessPool, id)
 		noerr(t, err)
 
-		sessStartingTransaction, err := session.NewClientSession(sessPool, id, session.Explicit)
+		sessStartingTransaction, err := session.NewClientSession(sessPool, id)
 		noerr(t, err)
 		err = sessStartingTransaction.StartTransaction(nil)
 		noerr(t, err)
 
-		sessInProgressTransaction, err := session.NewClientSession(sessPool, id, session.Explicit)
+		sessInProgressTransaction, err := session.NewClientSession(sessPool, id)
 		noerr(t, err)
 		err = sessInProgressTransaction.StartTransaction(nil)
 		noerr(t, err)
@@ -177,40 +184,6 @@ func TestOperation(t *testing.T) {
 			})
 		}
 	})
-	t.Run("roundTrip", func(t *testing.T) {
-		testCases := []struct {
-			name    string
-			conn    *mockConnection
-			paramWM []byte // parameter wire message
-			wantWM  []byte // wire message that should be returned
-			wantErr error  // error that should be returned
-		}{
-			{
-				"returns write error",
-				&mockConnection{rWriteErr: errors.New("write error")},
-				nil, nil,
-				Error{Message: "write error", Labels: []string{TransientTransactionError, NetworkError}},
-			},
-			{
-				"returns read error",
-				&mockConnection{rReadErr: errors.New("read error")},
-				nil, nil,
-				Error{Message: "read error", Labels: []string{TransientTransactionError, NetworkError}},
-			},
-		}
-
-		for _, tc := range testCases {
-			t.Run(tc.name, func(t *testing.T) {
-				gotWM, gotErr := Operation{}.roundTrip(context.Background(), tc.conn, tc.paramWM)
-				if !bytes.Equal(gotWM, tc.wantWM) {
-					t.Errorf("Returned wire messages are not equal. got %v; want %v", gotWM, tc.wantWM)
-				}
-				if !cmp.Equal(gotErr, tc.wantErr, cmp.Comparer(compareErrors)) {
-					t.Errorf("Returned error is not equal to expected error. got %v; want %v", gotErr, tc.wantErr)
-				}
-			})
-		}
-	})
 	t.Run("addReadConcern", func(t *testing.T) {
 		majorityRc := bsoncore.AppendDocumentElement(nil, "readConcern", bsoncore.BuildDocument(nil,
 			bsoncore.AppendStringElement(nil, "level", "majority"),
@@ -263,7 +236,7 @@ func TestOperation(t *testing.T) {
 			id, err := uuid.New()
 			noerr(t, err)
 
-			sess, err := session.NewClientSession(sessPool, id, session.Explicit)
+			sess, err := session.NewClientSession(sessPool, id)
 			noerr(t, err)
 			err = sess.AdvanceClusterTime(older)
 			noerr(t, err)
@@ -276,6 +249,76 @@ func TestOperation(t *testing.T) {
 			}
 		})
 	})
+	t.Run("calculateMaxTimeMS", func(t *testing.T) {
+		timeout := 5 * time.Second
+		maxTime := 2 * time.Second
+		negMaxTime := -2 * time.Second
+		shortRTT := 50 * time.Millisecond
+		longRTT := 10 * time.Second
+		timeoutCtx, cancel := internal.MakeTimeoutContext(context.Background(), timeout)
+		defer cancel()
+
+		testCases := []struct {
+			name  string
+			op    Operation
+			ctx   context.Context
+			rtt90 time.Duration
+			want  uint64
+			err   error
+		}{
+			{
+				name:  "uses context deadline and rtt90 with timeout",
+				op:    Operation{MaxTime: &maxTime},
+				ctx:   timeoutCtx,
+				rtt90: shortRTT,
+				want:  5000,
+				err:   nil,
+			},
+			{
+				name:  "uses MaxTime without timeout",
+				op:    Operation{MaxTime: &maxTime},
+				ctx:   context.Background(),
+				rtt90: longRTT,
+				want:  2000,
+				err:   nil,
+			},
+			{
+				name:  "errors when remaining timeout is less than rtt90",
+				op:    Operation{MaxTime: &maxTime},
+				ctx:   timeoutCtx,
+				rtt90: timeout,
+				want:  0,
+				err:   ErrDeadlineWouldBeExceeded,
+			},
+			{
+				name:  "errors when MaxTime is negative",
+				op:    Operation{MaxTime: &negMaxTime},
+				ctx:   context.Background(),
+				rtt90: longRTT,
+				want:  0,
+				err:   ErrNegativeMaxTime,
+			},
+		}
+		for _, tc := range testCases {
+			// Capture test-case for parallel sub-test.
+			tc := tc
+			t.Run(tc.name, func(t *testing.T) {
+				t.Parallel()
+
+				got, err := tc.op.calculateMaxTimeMS(tc.ctx, tc.rtt90, "")
+
+				// Assert that the calculated maxTimeMS is less than or equal to the expected value. A few
+				// milliseconds will have elapsed toward the context deadline, and (remainingTimeout
+				// - rtt90) will be slightly smaller than the expected value.
+				if got > tc.want {
+					t.Errorf("maxTimeMS value higher than expected. got %v; wanted at most %v", got, tc.want)
+				}
+				if !errors.Is(err, tc.err) {
+					t.Errorf("error values do not match. got %v; want %v", err, tc.err)
+				}
+			})
+		}
+	})
 	t.Run("updateClusterTimes", func(t *testing.T) {
 		clustertime := bsoncore.BuildDocumentFromElements(nil,
 			bsoncore.AppendDocumentElement(nil, "$clusterTime", bsoncore.BuildDocumentFromElements(nil,
@@ -288,7 +331,7 @@ func TestOperation(t *testing.T) {
 		id, err := uuid.New()
 		noerr(t, err)
 
-		sess, err := session.NewClientSession(sessPool, id, session.Explicit)
+		sess, err := session.NewClientSession(sessPool, id)
 		noerr(t, err)
 		Operation{Client: sess, Clock: clusterClock}.updateClusterTimes(clustertime)
 
@@ -310,7 +353,7 @@ func TestOperation(t *testing.T) {
 		id, err := uuid.New()
 		noerr(t, err)
 
-		sess, err := session.NewClientSession(sessPool, id, session.Explicit)
+		sess, err := session.NewClientSession(sessPool, id)
 		noerr(t, err)
 		if sess.OperationTime != nil {
 			t.Fatal("OperationTime should not be set on new session.")
@@ -524,7 +567,7 @@ func TestOperation(t *testing.T) {
 						Kind: tc.server,
 					},
 				}
-				wm, _, err := op.createQueryWireMessage(wm, desc)
+				wm, _, err := op.createQueryWireMessage(0, wm, desc)
 				noerr(t, err)
 
 				// We know where the $query would be within the OP_QUERY, so we'll just index into there.
@@ -544,7 +587,7 @@ func TestOperation(t *testing.T) {
 			conn := &mockConnection{
 				rStreaming: false,
 			}
-			err := Operation{}.ExecuteExhaust(context.TODO(), conn, nil)
+			err := Operation{}.ExecuteExhaust(context.TODO(), conn)
 			assert.NotNil(t, err, "expected error, got nil")
 		})
 	})
@@ -556,7 +599,7 @@ func TestOperation(t *testing.T) {
 		serverResponseDoc := bsoncore.BuildDocumentFromElements(nil,
 			bsoncore.AppendInt32Element(nil, "ok", 1),
 		)
-		nonStreamingResponse := createExhaustServerResponse(t, serverResponseDoc, false)
+		nonStreamingResponse := createExhaustServerResponse(serverResponseDoc, false)
 
 		// Create a connection that reports that it cannot stream messages.
 		conn := &mockConnection{
@@ -575,7 +618,7 @@ func TestOperation(t *testing.T) {
 			Database:   "admin",
 			Deployment: SingleConnectionDeployment{conn},
 		}
-		err := op.Execute(context.TODO(), nil)
+		err := op.Execute(context.TODO())
 		assert.Nil(t, err, "Execute error: %v", err)
 
 		// The wire message sent to the server should not have exhaustAllowed=true. After execution, the connection
@@ -584,10 +627,10 @@ func TestOperation(t *testing.T) {
 		assert.False(t, conn.CurrentlyStreaming(), "expected CurrentlyStreaming to be false")
 
 		// Modify the connection to report that it can stream and create a new server response with moreToCome=true.
-		streamingResponse := createExhaustServerResponse(t, serverResponseDoc, true)
+		streamingResponse := createExhaustServerResponse(serverResponseDoc, true)
 		conn.rReadWM = streamingResponse
 		conn.rCanStream = true
-		err = op.Execute(context.TODO(), nil)
+		err = op.Execute(context.TODO())
 		assert.Nil(t, err, "Execute error: %v", err)
 		assertExhaustAllowedSet(t, conn.pWriteWM, true)
 		assert.True(t, conn.CurrentlyStreaming(), "expected CurrentlyStreaming to be true")
@@ -595,13 +638,55 @@ func TestOperation(t *testing.T) {
 		// Reset the server response and go through ExecuteExhaust to mimic streaming the next response. After
 		// execution, the connection should still be in a streaming state.
 		conn.rReadWM = streamingResponse
-		err = op.ExecuteExhaust(context.TODO(), conn, nil)
+		err = op.ExecuteExhaust(context.TODO(), conn)
 		assert.Nil(t, err, "ExecuteExhaust error: %v", err)
 		assert.True(t, conn.CurrentlyStreaming(), "expected CurrentlyStreaming to be true")
 	})
+	t.Run("context deadline exceeded not marked as TransientTransactionError", func(t *testing.T) {
+		conn := new(mockConnection)
+		// Create a context that's already timed out.
+		ctx, cancel := context.WithDeadline(context.Background(), time.Unix(893934480, 0))
+		defer cancel()
+
+		op := Operation{
+			Database:   "foobar",
+			Deployment: SingleConnectionDeployment{C: conn},
+			CommandFn: func(dst []byte, desc description.SelectedServer) ([]byte, error) {
+				dst = bsoncore.AppendInt32Element(dst, "ping", 1)
+				return dst, nil
+			},
+		}
+
+		err := op.Execute(ctx)
+		assert.NotNil(t, err, "expected an error from Execute(), got nil")
+		// Assert that error is just context deadline exceeded and is therefore not a driver.Error marked
+		// with the TransientTransactionError label.
+		assert.Equal(t, err, context.DeadlineExceeded, "expected context.DeadlineExceeded error, got %v", err)
+	})
+	t.Run("canceled context not marked as TransientTransactionError", func(t *testing.T) {
+		conn := new(mockConnection)
+		// Create a context and cancel it immediately.
+		ctx, cancel := context.WithCancel(context.Background())
+		cancel()
+
+		op := Operation{
+			Database:   "foobar",
+			Deployment: SingleConnectionDeployment{C: conn},
+			CommandFn: func(dst []byte, desc description.SelectedServer) ([]byte, error) {
+				dst = bsoncore.AppendInt32Element(dst, "ping", 1)
+				return dst, nil
+			},
+		}
+
+		err := op.Execute(ctx)
+		assert.NotNil(t, err, "expected an error from Execute(), got nil")
+		// Assert that error is just context canceled and is therefore not a driver.Error marked with
+		// the TransientTransactionError label.
+		assert.Equal(t, err, context.Canceled, "expected context.Canceled error, got %v", err)
+	})
 }
 
-func createExhaustServerResponse(t *testing.T, response bsoncore.Document, moreToCome bool) []byte {
+func createExhaustServerResponse(response bsoncore.Document, moreToCome bool) []byte {
 	idx, wm := wiremessage.AppendHeaderStart(nil, 0, wiremessage.CurrentRequestID()+1, wiremessage.OpMsg)
 	var flags wiremessage.MsgFlag
 	if moreToCome {
@@ -640,7 +725,7 @@ type mockDeployment struct {
 	}
 }
 
-func (m *mockDeployment) SelectServer(ctx context.Context, desc description.ServerSelector) (Server, error) {
+func (m *mockDeployment) SelectServer(_ context.Context, desc description.ServerSelector) (Server, error) {
 	m.params.selector = desc
 	return m.returns.server, m.returns.err
 }
@@ -656,7 +741,6 @@ func (m *mockServerSelector) SelectServer(description.Topology, []description.Se
 type mockConnection struct {
 	// parameters
 	pWriteWM []byte
-	pReadDst []byte
 
 	// returns
 	rWriteErr     error
@@ -665,7 +749,7 @@ type mockConnection struct {
 	rDesc         description.Server
 	rCloseErr     error
 	rID           string
-	rServerConnID *int32
+	rServerConnID *int64
 	rAddr         address.Address
 	rCanStream    bool
 	rStreaming    bool
@@ -674,19 +758,156 @@ type mockConnection struct {
 func (m *mockConnection) Description() description.Server { return m.rDesc }
 func (m *mockConnection) Close() error                    { return m.rCloseErr }
 func (m *mockConnection) ID() string                      { return m.rID }
-func (m *mockConnection) ServerConnectionID() *int32      { return m.rServerConnID }
+func (m *mockConnection) ServerConnectionID() *int64      { return m.rServerConnID }
 func (m *mockConnection) Address() address.Address        { return m.rAddr }
 func (m *mockConnection) SupportsStreaming() bool         { return m.rCanStream }
 func (m *mockConnection) CurrentlyStreaming() bool        { return m.rStreaming }
 func (m *mockConnection) SetStreaming(streaming bool)     { m.rStreaming = streaming }
 func (m *mockConnection) Stale() bool                     { return false }
 
+// TODO:(GODRIVER-2824) replace return type with int64.
+func (m *mockConnection) DriverConnectionID() uint64 { return 0 }
+
 func (m *mockConnection) WriteWireMessage(_ context.Context, wm []byte) error {
 	m.pWriteWM = wm
 	return m.rWriteErr
 }
 
-func (m *mockConnection) ReadWireMessage(_ context.Context, dst []byte) ([]byte, error) {
-	m.pReadDst = dst
+func (m *mockConnection) ReadWireMessage(_ context.Context) ([]byte, error) {
 	return m.rReadWM, m.rReadErr
 }
+
+type retryableError struct {
+	error
+}
+
+func (retryableError) Retryable() bool { return true }
+
+var _ RetryablePoolError = retryableError{}
+
+// mockRetryServer is used to test retry of connection checkout. Returns a retryable error from
+// Connection().
+type mockRetryServer struct {
+	numCallsToConnection int
+}
+
+// Connection records the number of calls and returns retryable errors until the provided context
+// times out or is cancelled, then returns the context error.
+func (ms *mockRetryServer) Connection(ctx context.Context) (Connection, error) {
+	ms.numCallsToConnection++
+
+	if ctx.Err() != nil {
+		return nil, ctx.Err()
+	}
+
+	time.Sleep(1 * time.Millisecond)
+	return nil, retryableError{error: errors.New("test error")}
+}
+
+func (ms *mockRetryServer) RTTMonitor() RTTMonitor {
+	return &internal.ZeroRTTMonitor{}
+}
+
+func TestRetry(t *testing.T) {
+	t.Run("retries multiple times with RetryContext", func(t *testing.T) {
+		d := new(mockDeployment)
+		ms := new(mockRetryServer)
+		d.returns.server = ms
+
+		ctx, cancel := context.WithTimeout(context.Background(), 100*time.Millisecond)
+		defer cancel()
+
+		retry := RetryContext
+		err := Operation{
+			CommandFn:  func([]byte, description.SelectedServer) ([]byte, error) { return nil, nil },
+			Deployment: d,
+			Database:   "testing",
+			RetryMode:  &retry,
+			Type:       Read,
+		}.Execute(ctx)
+		assert.NotNil(t, err, "expected an error from Execute()")
+
+		// Expect Connection() to be called at least 3 times. The first call is the initial attempt
+		// to run the operation and the second is the retry. The third indicates that we retried
+		// more than once, which is the behavior we want to assert.
+		assert.True(t,
+			ms.numCallsToConnection >= 3,
+			"expected Connection() to be called at least 3 times")
+
+		deadline, _ := ctx.Deadline()
+		assert.True(t,
+			time.Now().After(deadline),
+			"expected operation to complete only after the context deadline is exceeded")
+	})
+}
+
+func TestConvertI64PtrToI32Ptr(t *testing.T) {
+	t.Parallel()
+
+	newI64 := func(i64 int64) *int64 { return &i64 }
+	newI32 := func(i32 int32) *int32 { return &i32 }
+
+	tests := []struct {
+		name string
+		i64  *int64
+		want *int32
+	}{
+		{
+			name: "empty",
+			want: nil,
+		},
+		{
+			name: "in bounds",
+			i64:  newI64(1),
+			want: newI32(1),
+		},
+		{
+			name: "out of bounds negative",
+			i64:  newI64(math.MinInt32 - 1),
+		},
+		{
+			name: "out of bounds positive",
+			i64:  newI64(math.MaxInt32 + 1),
+		},
+		{
+			name: "exact min int32",
+			i64:  newI64(math.MinInt32),
+			want: newI32(math.MinInt32),
+		},
+		{
+			name: "exact max int32",
+			i64:  newI64(math.MaxInt32),
+			want: newI32(math.MaxInt32),
+		},
+	}
+
+	for _, test := range tests {
+		test := test
+
+		t.Run(test.name, func(t *testing.T) {
+			t.Parallel()
+
+			got := convertInt64PtrToInt32Ptr(test.i64)
+			assert.Equal(t, test.want, got)
+		})
+	}
+}
+
+func TestDecodeOpReply(t *testing.T) {
+	t.Parallel()
+
+	// GODRIVER-2869: Prevent infinite loop caused by malformatted wiremessage with length of 0.
+	t.Run("malformatted wiremessage with length of 0", func(t *testing.T) {
+		t.Parallel()
+
+		var wm []byte
+		wm = wiremessage.AppendReplyFlags(wm, 0)
+		wm = wiremessage.AppendReplyCursorID(wm, int64(0))
+		wm = wiremessage.AppendReplyStartingFrom(wm, 0)
+		wm = wiremessage.AppendReplyNumberReturned(wm, 0)
+		idx, wm := bsoncore.ReserveLength(wm)
+		wm = bsoncore.UpdateLength(wm, idx, 0)
+		reply := Operation{}.decodeOpReply(wm)
+		assert.Equal(t, []bsoncore.Document(nil), reply.documents)
+	})
+}
