@@ -19,13 +19,13 @@ import (
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/event"
-	"go.mongodb.org/mongo-driver/internal/testutil/assert"
-	testhelpers "go.mongodb.org/mongo-driver/internal/testutil/helpers"
+	"go.mongodb.org/mongo-driver/internal/assert"
+	"go.mongodb.org/mongo-driver/internal/testutil/helpers"
 	"go.mongodb.org/mongo-driver/mongo/address"
 	"go.mongodb.org/mongo-driver/mongo/description"
+	"go.mongodb.org/mongo-driver/mongo/options"
 	"go.mongodb.org/mongo-driver/x/bsonx/bsoncore"
 	"go.mongodb.org/mongo-driver/x/mongo/driver"
-	"go.mongodb.org/mongo-driver/x/mongo/driver/connstring"
 )
 
 type response struct {
@@ -203,7 +203,7 @@ func serverClosed(e *event.ServerClosedEvent) {
 	lock.Unlock()
 }
 
-const testsDir string = "../../../../data/server-discovery-and-monitoring/"
+const testsDir string = "../../../../testdata/server-discovery-and-monitoring/"
 
 var publishedEvents []interface{}
 var lock sync.Mutex
@@ -222,9 +222,6 @@ func (r *response) UnmarshalBSON(buf []byte) error {
 }
 
 func setUpTopology(t *testing.T, uri string) *Topology {
-	cs, err := connstring.ParseAndValidate(uri)
-	assert.Nil(t, err, "Parse error: %v", err)
-
 	sdam := &event.ServerMonitor{
 		ServerDescriptionChanged:   serverDescriptionChanged,
 		ServerOpening:              serverOpening,
@@ -233,25 +230,14 @@ func setUpTopology(t *testing.T, uri string) *Topology {
 		ServerClosed:               serverClosed,
 	}
 
+	cfg, err := NewConfig(options.Client().ApplyURI(uri).SetServerMonitor(sdam), nil)
+	assert.Nil(t, err, "error constructing topology config: %v", err)
+
 	// Disable server monitoring because the hosts in the SDAM spec tests don't actually exist, so the server monitor
 	// can race with the test and mark the server Unknown when it fails to connect, which causes tests to fail.
-	serverOpts := []ServerOption{
-		withMonitoringDisabled(func(bool) bool {
-			return true
-		}),
-		WithServerMonitor(func(*event.ServerMonitor) *event.ServerMonitor { return sdam }),
-	}
-	topo, err := New(
-		WithConnString(func(connstring.ConnString) connstring.ConnString {
-			return cs
-		}),
-		WithServerOptions(func(opts ...ServerOption) []ServerOption {
-			return append(opts, serverOpts...)
-		}),
-		WithTopologyServerMonitor(func(*event.ServerMonitor) *event.ServerMonitor {
-			return sdam
-		}),
-	)
+	cfg.ServerOpts = append(cfg.ServerOpts, withMonitoringDisabled(func(bool) bool { return true }))
+
+	topo, err := New(cfg)
 	assert.Nil(t, err, "topology.New error: %v", err)
 
 	err = topo.Connect()
@@ -570,8 +556,174 @@ func runTest(t *testing.T, directory string, filename string) {
 // Test case for all SDAM spec tests.
 func TestSDAMSpec(t *testing.T) {
 	for _, subdir := range []string{"single", "rs", "sharded", "load-balanced", "errors", "monitoring"} {
-		for _, file := range testhelpers.FindJSONFilesInDir(t, path.Join(testsDir, subdir)) {
+		for _, file := range helpers.FindJSONFilesInDir(t, path.Join(testsDir, subdir)) {
 			runTest(t, subdir, file)
 		}
 	}
 }
+
+func TestHasStalePrimary(t *testing.T) {
+	t.Parallel()
+
+	t.Run("WV17 SEI EQ MEI and SSV LT MSV", func(t *testing.T) {
+		t.Parallel()
+
+		srv := description.Server{
+			WireVersion: &description.VersionRange{Min: 17, Max: 17},
+			ElectionID:  primitive.NewObjectIDFromTimestamp(time.Now()),
+			SetVersion:  1,
+		}
+
+		fsm := fsm{
+			maxElectionID: srv.ElectionID,
+			maxSetVersion: 2,
+		}
+
+		boolVal := hasStalePrimary(fsm, srv)
+		assert.True(t, boolVal, "expected true, got false")
+	})
+
+	t.Run("WV17 SEI EQ MEI and SSV GT MSV", func(t *testing.T) {
+		t.Parallel()
+
+		srv := description.Server{
+			WireVersion: &description.VersionRange{Min: 17, Max: 17},
+			ElectionID:  primitive.NewObjectIDFromTimestamp(time.Now()),
+			SetVersion:  2,
+		}
+
+		fsm := fsm{
+			maxElectionID: srv.ElectionID,
+			maxSetVersion: 1,
+		}
+
+		boolVal := hasStalePrimary(fsm, srv)
+		assert.False(t, boolVal, "expected false, got true")
+	})
+
+	t.Run("WV17 SEI EQ MEI and SSV EQ MSV", func(t *testing.T) {
+		t.Parallel()
+
+		srv := description.Server{
+			WireVersion: &description.VersionRange{Min: 17, Max: 17},
+			ElectionID:  primitive.NewObjectIDFromTimestamp(time.Now()),
+			SetVersion:  1,
+		}
+
+		fsm := fsm{
+			maxElectionID: srv.ElectionID,
+			maxSetVersion: 1,
+		}
+
+		boolVal := hasStalePrimary(fsm, srv)
+		assert.False(t, boolVal, "expected false, got true")
+	})
+
+	t.Run("WV17 SEI GT MEI and SSV LT MSV", func(t *testing.T) {
+		t.Parallel()
+
+		fsm := fsm{
+			maxElectionID: primitive.NewObjectIDFromTimestamp(time.Now()),
+			maxSetVersion: 2,
+		}
+
+		srv := description.Server{
+			WireVersion: &description.VersionRange{Min: 17, Max: 17},
+			ElectionID:  primitive.NewObjectIDFromTimestamp(time.Now().Add(time.Second)),
+			SetVersion:  1,
+		}
+
+		boolVal := hasStalePrimary(fsm, srv)
+		assert.False(t, boolVal, "expected false, got true")
+	})
+
+	t.Run("WV17 SEI GT MEI and SSV GT MSV", func(t *testing.T) {
+		t.Parallel()
+
+		fsm := fsm{
+			maxElectionID: primitive.NewObjectIDFromTimestamp(time.Now()),
+			maxSetVersion: 1,
+		}
+
+		srv := description.Server{
+			WireVersion: &description.VersionRange{Min: 17, Max: 17},
+			ElectionID:  primitive.NewObjectIDFromTimestamp(time.Now().Add(time.Second)),
+			SetVersion:  2,
+		}
+
+		boolVal := hasStalePrimary(fsm, srv)
+		assert.False(t, boolVal, "expected false, got true")
+	})
+
+	t.Run("WV17 SEI GT MEI and SSV EQ MSV", func(t *testing.T) {
+		t.Parallel()
+
+		fsm := fsm{
+			maxElectionID: primitive.NewObjectIDFromTimestamp(time.Now()),
+			maxSetVersion: 1,
+		}
+
+		srv := description.Server{
+			WireVersion: &description.VersionRange{Min: 17, Max: 17},
+			ElectionID:  primitive.NewObjectIDFromTimestamp(time.Now().Add(time.Second)),
+			SetVersion:  1,
+		}
+
+		boolVal := hasStalePrimary(fsm, srv)
+		assert.False(t, boolVal, "expected false, got true")
+	})
+
+	t.Run("WV17 SEI LT MEI and SSV LT MSV", func(t *testing.T) {
+		t.Parallel()
+
+		srv := description.Server{
+			WireVersion: &description.VersionRange{Min: 17, Max: 17},
+			ElectionID:  primitive.NewObjectIDFromTimestamp(time.Now().Add(-time.Second)),
+			SetVersion:  1,
+		}
+
+		fsm := fsm{
+			maxElectionID: primitive.NewObjectIDFromTimestamp(time.Now()),
+			maxSetVersion: 2,
+		}
+
+		boolVal := hasStalePrimary(fsm, srv)
+		assert.True(t, boolVal, "expected true, got false")
+	})
+
+	t.Run("WV17 SEI LT MEI and SSV GT MSV", func(t *testing.T) {
+		t.Parallel()
+
+		srv := description.Server{
+			WireVersion: &description.VersionRange{Min: 17, Max: 17},
+			ElectionID:  primitive.NewObjectIDFromTimestamp(time.Now().Add(-time.Second)),
+			SetVersion:  2,
+		}
+
+		fsm := fsm{
+			maxElectionID: primitive.NewObjectIDFromTimestamp(time.Now()),
+			maxSetVersion: 1,
+		}
+
+		boolVal := hasStalePrimary(fsm, srv)
+		assert.True(t, boolVal, "expected true, got false")
+	})
+
+	t.Run("WV17 SEI LT MEI and SSV EQ MSV", func(t *testing.T) {
+		t.Parallel()
+
+		srv := description.Server{
+			WireVersion: &description.VersionRange{Min: 17, Max: 17},
+			ElectionID:  primitive.NewObjectIDFromTimestamp(time.Now().Add(-time.Second)),
+			SetVersion:  1,
+		}
+
+		fsm := fsm{
+			maxElectionID: primitive.NewObjectIDFromTimestamp(time.Now()),
+			maxSetVersion: 1,
+		}
+
+		boolVal := hasStalePrimary(fsm, srv)
+		assert.True(t, boolVal, "expected true, got false")
+	})
+}
